﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;

namespace AgileCompass.Attributes
{
    [AttributeUsage(AttributeTargets.Property, Inherited = false, AllowMultiple = true)]
    public class ExpressionTypeAttribute : Attribute
    {
        public ExpressionType ExpressionType { get; set; }
        public string MethodName { get; set; }
        public MethodInfo MethodInfo { get; set; }

        public ExpressionTypeAttribute(ExpressionType type, string methodName)
        {
            this.ExpressionType = type;
            this.MethodName = methodName;
        }
        public ExpressionTypeAttribute(ExpressionType type)
        {
            this.ExpressionType = type;
        }

        public static ExpressionTypeAttribute Parse(PropertyInfo property)
        {
            var obj = property.GetCustomAttributes().Where(x => x.GetType() == typeof(ExpressionTypeAttribute)).FirstOrDefault() as ExpressionTypeAttribute;
            if (obj == null)
                obj = ExpressionTypeContainer.Container[property.PropertyType.FullName];

            if (obj.ExpressionType == ExpressionType.Call)
                obj.MethodInfo = property.PropertyType.GetMethods().Where(x => x.Name == obj.MethodName).FirstOrDefault();

            return obj;
        }
    }

    public static class ExpressionTypeContainer
    {
        private static Dictionary<string, ExpressionTypeAttribute> container;
        public static Dictionary<string, ExpressionTypeAttribute> Container
        {
            get
            {
                return container = container ??
                    Helper.ReferenceRepository<Dictionary<string, ExpressionTypeAttribute>>.Get("PublicExpressionTypeContainer") ??
                    Create();
            }
        }

        private static Dictionary<string, ExpressionTypeAttribute> Create()
        {
            var obj = new Dictionary<string, ExpressionTypeAttribute>();
            obj.Add(typeof(DateTime).FullName, new ExpressionTypeAttribute(ExpressionType.Equal));
            obj.Add(typeof(TimeSpan).FullName, new ExpressionTypeAttribute(ExpressionType.Equal));
            obj.Add(typeof(string).FullName, new ExpressionTypeAttribute(ExpressionType.Call, "Contains"));
            obj.Add(typeof(int).FullName, new ExpressionTypeAttribute(ExpressionType.Equal));
            obj.Add(typeof(long).FullName, new ExpressionTypeAttribute(ExpressionType.Equal));
            obj.Add(typeof(double).FullName, new ExpressionTypeAttribute(ExpressionType.Equal));
            obj.Add(typeof(float).FullName, new ExpressionTypeAttribute(ExpressionType.Equal));
            obj.Add(typeof(bool).FullName, new ExpressionTypeAttribute(ExpressionType.Equal));
            obj.Add(typeof(short).FullName, new ExpressionTypeAttribute(ExpressionType.Equal));
            obj.Add(typeof(decimal).FullName, new ExpressionTypeAttribute(ExpressionType.Equal));
            obj.Add(typeof(object).FullName, new ExpressionTypeAttribute(ExpressionType.Equal));
            obj.Add(typeof(int?).FullName, new ExpressionTypeAttribute(ExpressionType.Equal));
            obj.Add(typeof(long?).FullName, new ExpressionTypeAttribute(ExpressionType.Equal));
            obj.Add(typeof(double?).FullName, new ExpressionTypeAttribute(ExpressionType.Equal));
            obj.Add(typeof(float?).FullName, new ExpressionTypeAttribute(ExpressionType.Equal));
            obj.Add(typeof(bool?).FullName, new ExpressionTypeAttribute(ExpressionType.Equal));
            obj.Add(typeof(short?).FullName, new ExpressionTypeAttribute(ExpressionType.Equal));
            obj.Add(typeof(decimal?).FullName, new ExpressionTypeAttribute(ExpressionType.Equal));
            Helper.ReferenceRepository<Dictionary<string, ExpressionTypeAttribute>>.Set("PublicExpressionTypeContainer", obj);
            return obj;
        }
    }
}
