﻿using AgileCompass.Helper;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Net;
using System.Threading.Tasks;
using System.Web;
using System.Web.Mvc;

namespace AgileCompass.Controllers
{
    [VenomAuthorize(false)]
    public class AvaliacaoController : BaseController
    {
        // GET: Avaliacao
        public ActionResult Index(string search, int? page)
        {
            ViewBag.Mensagem = Session["Mensagem"];
            ViewBag.StatusAvaliacao = Session["StatusAvaliacao"];
            Session["Mensagem"] = Session["StatusAvaliacao"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var equipe = new Helper.AuthenticationManager().GetEquipe();
            IEnumerable<Models.Avaliacao> source = null;
            if (equipe.Admin)
                source = (string.IsNullOrEmpty(search)) ? new Models.Avaliacao().ToIEnumerable() : new Models.Avaliacao().SearchByString(search);
            else
                source = (string.IsNullOrEmpty(search)) ? new Models.Avaliacao().Where(x => x.EquipeId == equipe.Id).ToList() : new Models.Avaliacao().SearchByString(search).Where(x => x.EquipeId == equipe.Id).ToList();

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = page ?? 1;
            ViewBag.QtdPorPagina = qtd;

            source = source.Skip(((int)(page ?? 1) - 1) * qtd);
            return View(source.Take(qtd));
        }

        [HttpPost]
        public ActionResult Index(string search)
        {
            ViewBag.Mensagem = Session["Mensagem"];
            ViewBag.StatusAvaliacao = Session["StatusAvaliacao"];
            Session["Mensagem"] = Session["StatusAvaliacao"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var equipe = new Helper.AuthenticationManager().GetEquipe();
            IEnumerable<Models.Avaliacao> source = null;
            if (equipe.Admin)
                source = (string.IsNullOrEmpty(search)) ? new Models.Avaliacao().ToIEnumerable() : new Models.Avaliacao().SearchByString(search);
            else
                source = (string.IsNullOrEmpty(search)) ? new Models.Avaliacao().Where(x => x.EquipeId == equipe.Id).ToList() : new Models.Avaliacao().SearchByString(search).Where(x => x.EquipeId == equipe.Id).ToList();

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = 1;
            ViewBag.QtdPorPagina = qtd;

            return View(source.Take(qtd));
        }

        public ActionResult Create()
        {
            CarregarComboQuadro();
            return View(new Models.Avaliacao());
        }

        [HttpPost]
        public ActionResult Create(Models.Avaliacao model)
        {
            var equipeId = new Helper.AuthenticationManager().GetEquipe().Id;
            if (new Models.Avaliacao().Any(x => x.EquipeId == equipeId && !x.Status))
                return RedirectToAction("Index");

            model.EquipeId = equipeId;
            model.AvaliacaoItem = new Models.Item()
                .Where(x => x.Status && x.Resultado.Status && x.Resultado.Categoria.Status)
                .ToList()
                .Select(x => new Models.AvaliacaoItem { Avaliacao = model, Item = x }).ToList();

            model.Save();

            return RedirectToAction("Index");
        }

        public ActionResult Edit(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.Avaliacao { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            if (model.Status)
            {
                Session["Mensagem"] = Resource.MsgAvaliacaoFinalizada;
                Session["StatusAvaliacao"] = "danger";
                return RedirectToAction("Index");
            }
            var token = new Helper.AuthenticationManager().GetEquipe().TokenTrello;
            ViewBag.TrelloToken = token;
            if (string.IsNullOrEmpty(token))
                model.SincronizarTrello = false;
            ViewBag.Categoria = model.AvaliacaoItem.Select(x => x.Item.Resultado.Categoria).Distinct();
            return View(model);
        }

        [HttpPost]
        public ActionResult Edit(Models.Avaliacao model, bool finalizarAvaliacao, string tokenTrello)
        {
            try
            {
                model.Comentario = WebUtility.HtmlDecode(model.Comentario);
                model.Status = finalizarAvaliacao;
                model.Save();
                Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;

                if (!string.IsNullOrEmpty(tokenTrello) && model.SincronizarTrello)
                {
                    var equipe = new Helper.AuthenticationManager().GetEquipe();
                    equipe.TokenTrello = tokenTrello;
                    equipe.Save();

                    var avaliacao = new Models.Avaliacao { Id = model.Id }.Find();
                    var culture = System.Threading.Thread.CurrentThread.CurrentCulture;
                    var uiculture = System.Threading.Thread.CurrentThread.CurrentUICulture;
                    Task.Run(() =>
                    {
                        System.Threading.Thread.CurrentThread.CurrentCulture = culture;
                        System.Threading.Thread.CurrentThread.CurrentUICulture = uiculture;
                        new AvaliacaoTrelloHelper(ConfigurationManager.AppSettings["TrelloAppKey"], tokenTrello)
                        .SincronizarAvaliacao(avaliacao);
                    });
                }
                return RedirectToAction("Index");
            }
            catch
            {
                return RedirectToAction("Index");
            }
        }

        public ActionResult Delete(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.Avaliacao { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            if (model.Status)
            {
                Session["Mensagem"] = Resource.MsgAvaliacaoFinalizada;
                Session["StatusAvaliacao"] = "danger";
                return RedirectToAction("Index");
            }
            return View(model);
        }
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Delete(Models.Avaliacao model)
        {
            try
            {
                var context = model.GetDbContext();
                if (context.Avaliacao.Where(x => x.Id == model.Id && !x.Status).Count() > 0)
                {
                    var originalModel = new Models.Avaliacao { Id = model.Id }.Find();
                    originalModel.AvaliacaoItem.Clear();
                    context.SaveChanges();
                    originalModel.Delete();
                }
                else
                {
                    ViewBag.Mensagem = Resource.MsgExcluirAvaliacao;
                    return View(new Models.Avaliacao { Id = model.Id }.Find());
                }

                Session["Mensagem"] = Resource.MsgRegistroExcluirSucesso;
                return RedirectToAction("Index");
            }
            catch
            {
                return View(model);
            }
        }

        public ActionResult Details(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.Avaliacao { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            ViewBag.Categoria = model.AvaliacaoItem.Select(x => x.Item.Resultado.Categoria).Distinct();
            return View(model);
        }

        public ActionResult AvaliacaoItem(int? avaliacaoId, int? categoriaId)
        {
            if (avaliacaoId == null || categoriaId == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);

            var model = new Models.Avaliacao { Id = (int)avaliacaoId }.Find();
            var categoria = new Models.Categoria { Id = (int)categoriaId }.Find();
            if (model == null || categoria == null)
                return HttpNotFound();

            if (model.Status)
            {
                Session["Mensagem"] = Resource.MsgAvaliacaoFinalizada;
                Session["StatusAvaliacao"] = "danger";
                return RedirectToAction("Index");
            }

            model.AvaliacaoItem = model.AvaliacaoItem.Where(x => x.Item.Resultado.CategoriaId == categoriaId).OrderBy(x => x.Item.ResultadoId).ToList();
            ViewBag.Categoria = categoria;
            return View(model);
        }

        [HttpPost]
        public ActionResult AvaliacaoItem(Models.Avaliacao avaliacao)
        {
            var avaliacaoId = avaliacao.Id;
            var newItems = avaliacao.AvaliacaoItem.Select(x => { x.AvaliacaoId = avaliacao.Id; return x; }).ToArray();
            var oldItems = new Models.AvaliacaoItem().ToIEnumerable().Where(x => x.AvaliacaoId == avaliacaoId && newItems.Any(y => y.ItemId == x.ItemId)).ToList();
            foreach (var item in oldItems)
                item.Delete();
            foreach (var item in newItems)
                item.Save();

            var avaliacaotemp = new Models.Avaliacao { Id = avaliacao.Id }.Find();
            avaliacaotemp.JsonBoard = avaliacao.JsonBoard;
            avaliacaotemp.IdentityCard = avaliacao.IdentityCard;
            avaliacaotemp.Save();

            return RedirectToAction("Edit", new { id = avaliacao.Id });
        }

        public void CarregarTelaAvaliacao()
        {
            ViewBag.Categoria = new Models.Categoria().Where(x => x.Status).ToList();
        }

        void CarregarComboQuadro()
        {
            var data = new Models.Quadro().Where(x => x.Status).ToList();
            ViewBag.QuadroId = new SelectList(data, "Id", "IdiomaQuadroDefault.Nome");
        }
    }
}