﻿using System.Web.Mvc;
using System.Linq;
using System;
using System.Collections.Generic;
using System.Net;
using AgileCompass.Helper;
using System.Text;
using System.Configuration;

namespace AgileCompass.Controllers
{
    [VenomAuthorize(true)]
    public class CategoriaController : BaseController
    {
        // GET: Categoria
        public ActionResult Index(string search, int? page)
        {
            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = (string.IsNullOrEmpty(search)) ? new Models.Categoria().ToIEnumerable() : new Models.Categoria().SearchByString(search);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = page ?? 1;
            ViewBag.QtdPorPagina = qtd;

            source = source.Skip(((int)(page ?? 1) - 1) * qtd);
            return View(source.Take(qtd));
        }

        [HttpPost]
        public ActionResult Index(string search)
        {
            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = new Models.Categoria().SearchByString(search);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = 1;
            ViewBag.QtdPorPagina = qtd;

            return View(source.Take(qtd));
        }

        public ActionResult Create()
        {
            CarregarComboIdioma();
            return View(new Models.Categoria());
        }
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Models.Categoria model)
        {
            try
            {
                model.DataCadastro = DateTime.Now;
                if (ModelState.IsValid && model.Save())
                {
                    Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                    return RedirectToAction("Index");
                }

                model.IdiomaCategoria = CarregarItensIdioma(model.IdiomaCategoria);
                CarregarComboIdioma();
                return View(model);
            }
            catch (Exception)
            {
                model.IdiomaCategoria = CarregarItensIdioma(model.IdiomaCategoria);
                CarregarComboIdioma();
                return View(model);
            }
        }
        public ActionResult Edit(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.Categoria { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            CarregarComboIdioma();
            return View(model);
        }
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(Models.Categoria model)
        {
            try
            {
                if (!ModelState.IsValid)
                {
                    model.IdiomaCategoria = CarregarItensIdioma(model.IdiomaCategoria);
                    CarregarComboIdioma();
                    return View(model);
                }
                var items = new Models.IdiomaCategoria().Where(x => x.CategoriaId == model.Id).ToList();
                foreach (var item in items)
                    item.Delete();
                foreach (var item in model.IdiomaCategoria)
                    item.Save();

                model.Save();
                Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                return RedirectToAction("Index");
            }
            catch
            {
                model.IdiomaCategoria = CarregarItensIdioma(model.IdiomaCategoria);
                CarregarComboIdioma();
                return View(model);
            }
        }
        public ActionResult Delete(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.Categoria { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            return View(model);
        }
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Delete(Models.Categoria model)
        {
            try
            {
                if (!ModelState.IsValid)
                    return View(model);

                if ((model.GetDbContext()).Resultado.Where(x => x.CategoriaId == model.Id).Count() < 1)
                {
                    var items = new Models.IdiomaCategoria().Where(x => x.CategoriaId == model.Id).ToList();
                    foreach (var item in items)
                        item.Delete();

                    new Models.Categoria().Where(x => x.Id == model.Id).Single().Delete();
                }
                else
                {
                    ViewBag.Mensagem = Resource.MsgExcluirDependencia;
                    return View(new Models.Categoria { Id = model.Id }.Find());
                }
                Session["Mensagem"] = Resource.MsgRegistroExcluirSucesso;
                return RedirectToAction("Index");
            }
            catch
            {
                return View(model);
            }
        }
        public ActionResult Details(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.Categoria { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            return View(model);
        }

        [HttpGet]
        public FileResult ExportCSV()
        {
            var lista = new Models.Categoria().ToIEnumerable();
            var csv = "ID;Nome;Descrição;Data de Cadastro;Status\n";
            csv += lista.Aggregate(string.Empty,
                (seed, next) => seed +
                    next.Id.ToString() + ";" +
                    next.IdiomaCategoriaDefault.Nome + ";" +
                    next.IdiomaCategoriaDefault.Descricao + ";" +
                    next.DataCadastro + ";" +
                    next.Status +
                    "\n");

            var bytes = Encoding.GetEncoding("iso-8859-1").GetBytes(csv);
            var stream = new System.IO.MemoryStream(bytes);
            return File(stream, "text/csv", "DadosCategoria.csv");
        }

        // ### Helper Methods ###

        void CarregarComboIdioma()
        {
            var data = new Models.Idioma().Where(x => x.Status).ToList();
            ViewBag.IdiomaId = new SelectList(data, "Id", "Nome");
        }

        List<Models.IdiomaCategoria> CarregarItensIdioma(List<Models.IdiomaCategoria> source)
        {
            var data = new Models.Idioma().ToIEnumerable();
            source = source.Select(x => new Models.IdiomaCategoria
            {
                IdiomaId = x.IdiomaId,
                Nome = x.Nome,
                Descricao = x.Descricao,
                Idioma = data.Where(y => y.Id == x.IdiomaId).Single()
            }).ToList();
            return source;
        }
    }
}