﻿using System.Web.Mvc;
using System.Linq;
using System;
using System.Collections.Generic;
using System.Net;
using AgileCompass.Helper;
using System.Text;
using AgileCompass.Models;
using System.Configuration;

namespace AgileCompass.Controllers
{
    [VenomAuthorize(true)]
    public class ContatoController : BaseController
    {
        public ActionResult Index(string search, int? page)
        {
            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = (string.IsNullOrEmpty(search)) ? new Models.Contato().ToIEnumerable() : new Models.Contato().SearchByString(search);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = page ?? 1;
            ViewBag.QtdPorPagina = qtd;

            source = source.Skip(((int)(page ?? 1) - 1) * qtd);
            return View(source.Take(qtd));
        }

        [HttpPost]
        public ActionResult Index(string search)
        {
            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = new Models.Contato().SearchByString(search);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = 1;
            ViewBag.QtdPorPagina = qtd;

            return View(source.Take(qtd));
        }

        public ActionResult Answer(int? contatoId)
        {
            if (contatoId == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);

            var obj = new Models.ContatoResposta { ContatoId = (int)contatoId };
            obj.Contato = new Models.Contato { Id = (int)contatoId }.Find();
            if (obj.Contato == null)
                return HttpNotFound();
            return View(obj);
        }
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Answer(Models.ContatoResposta model)
        {
            try
            {
                if (!ModelState.IsValid)
                    return View(model);

                model.DataCadastro = DateTime.Now;
                model.Contato = new Models.Contato { Id = model.ContatoId }.Find();
                model.Save();
                Helper.EmailHelper.Send(new[] { model.Contato.Email }, Resource.EmailTituloContato + " - " + model.Assunto, Helper.EmailHelper.EmailContato(model), true);
                Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                return RedirectToAction("Index");
            }
            catch
            {
                return View(model);
            }
        }

        public ActionResult Details(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.Contato { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            return View(model);
        }

        [HttpGet]
        public FileResult ExportCSV()
        {
            var lista = new Models.Contato().ToIEnumerable();
            var csv = "ID;Nome;E-mail;Mensagem;Data de Cadastro;Resposta - Assunto;Resposta - Mensagem;Resposta - Data de Cadastro\n";
            csv += lista.Aggregate(string.Empty,
                (seed, next) => seed +
                    next.Id.ToString() + ";" +
                    next.Nome + ";" +
                    next.Email + ";" +
                    next.Mensagem + ";" +
                    next.DataCadastro + ";" +
                    (next.ContatoResposta.Count > 0 ? next.ContatoResposta[0].Assunto : string.Empty) + ";" +
                    (next.ContatoResposta.Count > 0 ? next.ContatoResposta[0].Mensagem : string.Empty) + ";" +
                    (next.ContatoResposta.Count > 0 ? next.ContatoResposta[0].DataCadastro.ToString() : string.Empty) +
                    "\n");

            var bytes = Encoding.GetEncoding("iso-8859-1").GetBytes(csv);
            var stream = new System.IO.MemoryStream(bytes);
            return File(stream, "text/csv", "DadosContato.csv");
        }
    }
}