﻿using System.Web.Mvc;
using System.Linq;
using System;
using System.Collections.Generic;
using System.Net;
using AgileCompass.Helper;
using System.Configuration;

namespace AgileCompass.Controllers
{
    [VenomAuthorize(true)]
    public class ConteudoController : BaseController
    {
        // GET: Conteúdo
        public ActionResult Index(string search, int? page)
        {
            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = (string.IsNullOrEmpty(search)) ? new Models.Conteudo().ToIEnumerable() : new Models.Conteudo().SearchByString(search);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = page ?? 1;
            ViewBag.QtdPorPagina = qtd;

            source = source.Skip(((int)(page ?? 1) - 1) * qtd);
            return View(source.Take(qtd));
        }

        [HttpPost]
        public ActionResult Index(string search)
        {
            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = new Models.Conteudo().SearchByString(search);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = 1;
            ViewBag.QtdPorPagina = qtd;

            return View(source.Take(qtd));
        }

        public ActionResult Create()
        {
            CarregarComboIdioma();
            CarregarRadioTipoConteudo();
            return View(new Models.Conteudo());
        }
        [HttpPost]
        [ValidateAntiForgeryToken]
        [ValidateInput(false)]
        public ActionResult Create(Models.Conteudo model, int tipoConteudoId)
        {
            try
            {
                model.IdiomaConteudo = CarregarItensIdioma(model.IdiomaConteudo);
                model.DataCadastro = DateTime.Now;
                model.TipoConteudo = new Models.TipoConteudo() { Id = (int)tipoConteudoId }.Find();
                if (ModelState.IsValid && model.Save())
                {
                    Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                    return RedirectToAction("Index");
                }

                model.IdiomaConteudo = CarregarItensIdioma(model.IdiomaConteudo);
                CarregarComboIdioma();
                CarregarRadioTipoConteudo();
                return View(model);
            }
            catch (Exception)
            {
                model.IdiomaConteudo = CarregarItensIdioma(model.IdiomaConteudo);
                CarregarComboIdioma();
                return View(model);
            }
        }
        public ActionResult Edit(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.Conteudo { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            CarregarComboIdioma();
            CarregarRadioTipoConteudo();
            return View(model);
        }
        [HttpPost]
        [ValidateAntiForgeryToken]
        [ValidateInput(false)]
        public ActionResult Edit(Models.Conteudo model, int tipoConteudoId)
        {
            try
            {
                model.TipoConteudo = new Models.TipoConteudo() { Id = (int)tipoConteudoId }.Find();
                if (!ModelState.IsValid)
                {
                    model.IdiomaConteudo = CarregarItensIdioma(model.IdiomaConteudo);
                    CarregarComboIdioma();
                    CarregarRadioTipoConteudo();
                    return View(model);
                }
                var items = new Models.IdiomaConteudo().Where(x => x.ConteudoId == model.Id).ToList();
                foreach (var item in items)
                    item.Delete();
                foreach (var item in model.IdiomaConteudo)
                    item.Save();

                model.Save();
                Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                return RedirectToAction("Index");
            }
            catch
            {
                model.IdiomaConteudo = CarregarItensIdioma(model.IdiomaConteudo);
                CarregarRadioTipoConteudo();
                CarregarComboIdioma();
                return View(model);
            }
        }
        public ActionResult Delete(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.Conteudo { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            return View(model);
        }
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Delete(Models.Conteudo model)
        {
            try
            {
                if (!ModelState.IsValid)
                    return View(model);

                var items = new Models.IdiomaConteudo().Where(x => x.ConteudoId == model.Id).ToList();
                foreach (var item in items)
                    item.Delete();

                new Models.Conteudo().Where(x => x.Id == model.Id).Single().Delete();
                Session["Mensagem"] = Resource.MsgRegistroExcluirSucesso;
                return RedirectToAction("Index");
            }
            catch
            {
                return View(model);
            }
        }
        public ActionResult Details(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.Conteudo { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            return View(model);
        }

        // ### Helper Methods ###

        void CarregarComboIdioma()
        {
            var data = new Models.Idioma().Where(x => x.Status).ToList();
            ViewBag.IdiomaId = new SelectList(data, "Id", "Nome");
        }
        void CarregarRadioTipoConteudo()
        {
            ViewBag.TipoConteudo = new Models.TipoConteudo().ToIEnumerable();
        }

        List<Models.IdiomaConteudo> CarregarItensIdioma(List<Models.IdiomaConteudo> source)
        {
            var data = new Models.Idioma().ToIEnumerable();
            source = source.Select(x => new Models.IdiomaConteudo
            {
                IdiomaId = x.IdiomaId,
                Titulo = x.Titulo,
                Subtitulo = x.Subtitulo,
                ConteudoValue = WebUtility.HtmlDecode(x.ConteudoValue),
                Idioma = data.Where(y => y.Id == x.IdiomaId).Single()
            }).ToList();
            return source;
        }
    }
}