﻿using AgileCompass.Helper;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Web.Mvc;

namespace AgileCompass.Controllers
{
    [VenomAuthorize(false)]
    public class EquipeController : BaseController
    {
        // GET: Equipe
        [VenomAuthorize(true)]
        public ActionResult Index(string search, int? page)
        {
            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = (string.IsNullOrEmpty(search)) ? new Models.Equipe().ToIEnumerable() : new Models.Equipe().SearchByString(search);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = page ?? 1;
            ViewBag.QtdPorPagina = qtd;

            source = source.Skip(((int)(page ?? 1) - 1) * qtd);
            return View(source.Take(qtd));
        }

        [HttpPost]
        [VenomAuthorize(true)]
        public ActionResult Index(string search)
        {
            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = new Models.Equipe().SearchByString(search);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = 1;
            ViewBag.QtdPorPagina = qtd;

            return View(source.Take(qtd));
        }

        [AllowAnonymous]
        public ActionResult Create()
        {
            CarregarTela();
            return View();
        }
        [HttpPost]
        [AllowAnonymous]
        public ActionResult Create(Models.Equipe model, int[] MetodosSelecionados, int[] PraticasSelecionadas, int[] TiposSoftwareSelecionados)
        {
            try
            {
                foreach (var item in MetodosSelecionados ?? new int[0])
                    model.Metodo.Add(new Models.Metodo { Id = item }.Find());
                foreach (var item in PraticasSelecionadas ?? new int[0])
                    model.Pratica.Add(new Models.Pratica { Id = item }.Find());
                foreach (var item in TiposSoftwareSelecionados ?? new int[0])
                    model.TipoSoftware.Add(new Models.TipoSoftware { Id = item }.Find());
                model.DataCadastro = DateTime.Now;
                model.Status = true;
                model.Admin = false;
                model.Senha = Helper.Cryptography.HashMD5(model.Senha);

                if (ModelState.IsValid && model.Save())
                {
                    Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                    return RedirectToAction("Index");
                }

                CarregarTela();
                return View(model);
            }
            catch (Exception)
            {
                CarregarTela();
                return View(model);
            }
        }

        public ActionResult Edit(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);

            var manager = new AgileCompass.Helper.AuthenticationManager();
            if (id == manager.GetEquipe().Id)
            {
                var model = new Models.Equipe { Id = (int)id }.Find();
                if (model == null)
                    return HttpNotFound();

                ViewBag.Mensagem = Session["Mensagem"];
                Session["Mensagem"] = string.Empty;
                CarregarTela(model);
                return View(model);
            }
            else
                return RedirectToAction("Index");
        }
        [HttpPost]
        public ActionResult Edit(Models.Equipe model, int[] MetodosSelecionados, int[] PraticasSelecionadas, int[] TiposSoftwareSelecionados)
        {
            try
            {
                if (!ModelState.IsValid)
                {
                    CarregarTela(model);
                    return View(model);
                }
                var context = model.GetDbContext();
                var originalModel = new Models.Equipe { Id = model.Id }.Find();
                originalModel.Metodo.Clear();
                originalModel.Pratica.Clear();
                originalModel.TipoSoftware.Clear();
                context.SaveChanges();

                context.Entry(originalModel).State = System.Data.Entity.EntityState.Detached;
                context.Equipe.Attach(model);
                context.Entry(model).State = System.Data.Entity.EntityState.Modified;

                foreach (var item in MetodosSelecionados ?? new int[0])
                    model.Metodo.Add(new Models.Metodo { Id = item }.Find());
                foreach (var item in PraticasSelecionadas ?? new int[0])
                    model.Pratica.Add(new Models.Pratica { Id = item }.Find());
                foreach (var item in TiposSoftwareSelecionados ?? new int[0])
                    model.TipoSoftware.Add(new Models.TipoSoftware { Id = item }.Find());

                model.Status = true;
                model.Admin = originalModel.Admin;
                model.Senha = Helper.Cryptography.HashMD5(model.Senha);
                model.Save();
                Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                return RedirectToAction("Edit", new { id = model.Id });
            }
            catch
            {
                CarregarTela(model);
                return View(model);
            }
        }

        [VenomAuthorize(true)]
        public ActionResult EditAdmin(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);

            var model = new Models.Equipe { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            CarregarTela(model);
            return View(model);
        }
        [HttpPost]
        [VenomAuthorize(true)]
        public ActionResult EditAdmin([Bind(Exclude = "Senha")]Models.Equipe model, int[] MetodosSelecionados, int[] PraticasSelecionadas, int[] TiposSoftwareSelecionados)
        {
            try
            {
                ModelState.Remove("Senha");
                if (!ModelState.IsValid)
                {
                    CarregarTela(model);
                    return View(model);
                }
                var context = model.GetDbContext();
                var originalModel = new Models.Equipe { Id = model.Id }.Find();
                originalModel.Metodo.Clear();
                originalModel.Pratica.Clear();
                originalModel.TipoSoftware.Clear();
                context.SaveChanges();

                context.Entry(originalModel).State = System.Data.Entity.EntityState.Detached;
                context.Equipe.Attach(model);
                context.Entry(model).State = System.Data.Entity.EntityState.Modified;

                foreach (var item in MetodosSelecionados ?? new int[0])
                    model.Metodo.Add(new Models.Metodo { Id = item }.Find());
                foreach (var item in PraticasSelecionadas ?? new int[0])
                    model.Pratica.Add(new Models.Pratica { Id = item }.Find());
                foreach (var item in TiposSoftwareSelecionados ?? new int[0])
                    model.TipoSoftware.Add(new Models.TipoSoftware { Id = item }.Find());

                model.Senha = originalModel.Senha;
                model.Save();
                Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                return RedirectToAction("Index");
            }
            catch
            {
                CarregarTela(model);
                return View(model);
            }
        }

        [VenomAuthorize(true)]
        public ActionResult Delete(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.Equipe { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            return View(model);
        }
        [HttpPost]
        [ValidateAntiForgeryToken]
        [VenomAuthorize(true)]
        public ActionResult Delete(Models.Equipe model)
        {
            try
            {
                var context = model.GetDbContext();
                if (context.Avaliacao.Where(x => x.EquipeId == model.Id).Count() < 1)
                {   
                    var originalModel = new Models.Equipe { Id = model.Id }.Find();
                    originalModel.Metodo.Clear();
                    originalModel.Pratica.Clear();
                    originalModel.TipoSoftware.Clear();
                    context.SaveChanges();
                    originalModel.Delete();
                }
                else
                {
                    ViewBag.Mensagem = Resource.MsgExcluirDependencia;
                    return View(new Models.Equipe { Id = model.Id }.Find());
                }

                Session["Mensagem"] = Resource.MsgRegistroExcluirSucesso;
                return RedirectToAction("Index");
            }
            catch
            {
                return View(model);
            }
        }

        [VenomAuthorize(true)]
        public ActionResult Details(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.Equipe { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            return View(model);
        }

        [AllowAnonymous]
        public ActionResult RecoverPassword()
        {
            return View();
        }
        [HttpPost]
        [AllowAnonymous]
        public ActionResult RecoverPassword(string txtEmail)
        {
            try
            {
                if (string.IsNullOrEmpty(txtEmail))
                {
                    ViewBag.Mensagem = Resource.MsgEmailObrigatorio;
                    return View();
                }

                Models.Equipe equipe = new Models.Equipe();
                var context = equipe.GetDbContext();
                if (context.Equipe.Where(x => x.Email == txtEmail).Count() < 1)
                {
                    ViewBag.Mensagem = Resource.MsgEmailNaoCadastrado;
                    return View();
                }

                string senha = (Path.GetRandomFileName()).Replace(".", "");
                equipe = context.Equipe.Where(x => x.Email == txtEmail).FirstOrDefault();
                equipe.Senha = Helper.Cryptography.HashMD5(senha);
                equipe.Save();
                Helper.EmailHelper.Send(new[] { equipe.Email }, Resource.EmailTituloRecuperarSenha, Helper.EmailHelper.EmailRecuperarSenha(equipe, senha), true);
                Session["Mensagem"] = Resource.MsgDadosEnviadosEmail;
                return RedirectToAction("Index", "Login");
            }
            catch
            {
                return View();
            }
        }

        [HttpGet]
        public FileResult ExportCSV()
        {
            var lista = new Models.Equipe().ToIEnumerable();
            var csv = "ID;Nome;E-mail;Número de Integrantes;Data Início de Utilização de Métodos Ágeis;Empresa;Idioma;País;Cidade;Métodos Utilizados;Práticas Utilizadas;Tipos de Software Utilizados;Data de Cadastro;Admin;Status\n";
            csv += lista.Aggregate(string.Empty,
                (seed, next) => seed +
                    next.Id.ToString() + ";" +
                    next.Nome + ";" +
                    next.Email + ";" +
                    next.NumIntegrante.ToString() + ";" +
                    next.DataInicioAgil + ";" +
                    next.Empresa + ";" +
                    next.IdiomaId.ToString() + ";" +
                    next.PaisId.ToString() + ";" +
                    next.Cidade + ";" +
                    string.Join("+", next.Metodo.Select(x => x.Id)) + ";" +
                    string.Join("+", next.Pratica.Select(x => x.Id)) + ";" +
                    string.Join("+", next.TipoSoftware.Select(x => x.Id)) + ";" +
                    next.DataCadastro + ";" +
                    next.Admin + ";" +
                    next.Status +
                    "\n");

            var bytes = Encoding.GetEncoding("iso-8859-1").GetBytes(csv);
            var stream = new System.IO.MemoryStream(bytes);
            return File(stream, "text/csv", "DadosEquipe.csv");
        }

        void CarregarTela(Models.Equipe equipe = null)
        {
            if (equipe == null)
            {
                CarregarComboIdioma();
                CarregarComboPais();
            }
            else
            {
                CarregarComboIdioma(equipe.IdiomaId);
                CarregarComboPais(equipe.PaisId);
            }
            CarregarListMetodo();
            CarregarListPratica();
            CarregarListTipoSoftware();
        }

        void CarregarComboIdioma(int? selected = null)
        {
            var data = new Models.Idioma().Where(x => x.Status).ToList();
            ViewBag.IdiomaId = new SelectList(data, "Id", "Nome", selected);
        }
        void CarregarComboPais(int? selected = null)
        {
            var data = new Models.Pais().ToIEnumerable();
            ViewBag.PaisId = new SelectList(data, "Id", "IdiomaPaisDefault.Nome", selected);
        }
        void CarregarListMetodo()
        {
            var data = new Models.Metodo().Where(x => x.Status).ToList();
            ViewBag.Metodo = new SelectList(data, "Id", "IdiomaMetodoDefault.Nome");
        }
        void CarregarListPratica()
        {
            var data = new Models.Pratica().Where(x => x.Status).ToList();
            ViewBag.Pratica = new SelectList(data, "Id", "IdiomaPraticaDefault.Nome");
        }
        void CarregarListTipoSoftware()
        {
            var data = new Models.TipoSoftware().Where(x => x.Status).ToList();
            ViewBag.TipoSoftware = new SelectList(data, "Id", "IdiomaTipoSoftwareDefault.Nome");
        }
    }
}