﻿using System.Web.Mvc;
using System.Threading.Tasks;
using System;
using System.Linq;
using System.Globalization;

namespace AgileCompass.Controllers
{
    public class HomeController : BaseController
    {
        public ActionResult Index()
        {
	    CarregarConteudo();
            return View();
        }

        public ActionResult Estatisticas()
        {
            var data = Models.ViewModel.EstatisticasViewModel.ListEstatisticas();
            return PartialView(data);
        }

        public ActionResult Idioma()
        {
            return PartialView(new Models.Idioma().Where(x => x.Status).ToList());
        }

        public ActionResult Categoria()
        {
            return PartialView(new Models.Categoria().Where(x => x.Status).ToList());
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Index(Models.ViewModel.LoginViewModel model)
        {
            if (!ModelState.IsValid)
                return RedirectToAction("Index");

            var equipe = new Models.Equipe { Email = model.Email, Senha = Helper.Cryptography.HashMD5(model.Senha) };
            var result = equipe.Autenticar();
            switch (result)
            {
                case En.Status.Sucesso:
                    new Helper.AuthenticationManager().SetAuthenticationEquipe(equipe);
                    return RedirectToAction("Index");
                case En.Status.Falha:
                default:
                    ModelState.AddModelError("", Resource.MsgLoginIncorreto);
                    return View(model);
            }
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Logout()
        {
            new Helper.AuthenticationManager().SignOut();
            return RedirectToAction("Index");
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Contato(Models.Contato model)
        {
            if (!ModelState.IsValid)
                return RedirectToAction("Index");

            var contato = new Models.Contato { Nome = model.Nome, Email = model.Email, Mensagem = model.Mensagem, DataCadastro = DateTime.Now };
            if (contato.Save())
                return RedirectToAction("Index");
            return RedirectToAction("Index");
        }

	    public void CarregarConteudo()
        {
            ViewBag.Conteudo = new Models.Conteudo().Where(x => x.Status).ToList();
        }

        public ActionResult MudarIdioma(string idioma, string url)
        {
            Session["Idioma"] = idioma;
            return Redirect(url);
        }
    }
}