﻿using AgileCompass.Helper;
using System;
using System.Configuration;
using System.Linq;
using System.Net;
using System.Text;
using System.Web.Mvc;

namespace AgileCompass.Controllers
{
    [VenomAuthorize(true)]
    public class IdiomaController : BaseController
    {
        // GET: Idioma
        public ActionResult Index(string search, int? page)
        {
            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = (string.IsNullOrEmpty(search)) ? new Models.Idioma().ToIEnumerable() : new Models.Idioma().SearchByString(search);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = page ?? 1;
            ViewBag.QtdPorPagina = qtd;

            source = source.Skip(((int)(page ?? 1) - 1) * qtd);
            return View(source.Take(qtd));
        }

        [HttpPost]
        public ActionResult Index(string search)
        {
            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = new Models.Idioma().SearchByString(search);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = 1;
            ViewBag.QtdPorPagina = qtd;

            return View(source.Take(qtd));
        }

        public ActionResult Create()
        {
            return View(new Models.Idioma());
        }
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Models.Idioma model)
        {
            try
            {
                if (!ModelState.IsValid)
                    return View(model);

                var context = model.GetDbContext();
                if (model.Padrao && model.Status)
                    context.Idioma.ToList().Select(x => { x.Padrao = false; return x; }).ToList();
                else
                    model.Padrao = (context.Idioma.Where(x => x.Padrao).Count() < 1);
                model.Save();
                Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                return RedirectToAction("Index");
            }
            catch (Exception)
            {
                return View(model);
            }
        }
        public ActionResult Edit(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.Idioma { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();
            return View(model);
        }
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(Models.Idioma model)
        {
            try
            {
                if (!ModelState.IsValid)
                    return View(model);

                var context = model.GetDbContext();
                if (model.Padrao && model.Status)
                    context.Idioma.ToList().Select(x => { x.Padrao = false; return x; }).ToList();
                else
                    model.Padrao = (context.Idioma.Where(x => x.Padrao).Count() < 1);

                model.Save();
                Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                return RedirectToAction("Index");
            }
            catch
            {
                return View(model);
            }
        }
        public ActionResult Delete(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.Idioma { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            return View(model);
        }
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Delete(Models.Idioma model)
        {
            try
            {
                var valida = (model.GetDbContext()).Categoria.Where(x => x.IdiomaCategoria.Any(y => y.IdiomaId == model.Id)).Count();
                valida += (model.GetDbContext()).Item.Where(x => x.IdiomaItem.Any(y => y.IdiomaId == model.Id)).Count();
                valida += (model.GetDbContext()).Metodo.Where(x => x.IdiomaMetodo.Any(y => y.IdiomaId == model.Id)).Count();
                valida += (model.GetDbContext()).Pais.Where(x => x.IdiomaPais.Any(y => y.IdiomaId == model.Id)).Count();
                valida += (model.GetDbContext()).Pratica.Where(x => x.IdiomaPratica.Any(y => y.IdiomaId == model.Id)).Count();
                valida += (model.GetDbContext()).Resultado.Where(x => x.IdiomaResultado.Any(y => y.IdiomaId == model.Id)).Count();
                valida += (model.GetDbContext()).TipoSoftware.Where(x => x.IdiomaTipoSoftware.Any(y => y.IdiomaId == model.Id)).Count();
                if (valida < 1)
                    new Models.Idioma().Where(x => x.Id == model.Id).Single().Delete();
                else
                {
                    ViewBag.Mensagem = Resource.MsgExcluirDependencia;
                    return View(new Models.Idioma { Id = model.Id }.Find());
                }

                Session["Mensagem"] = Resource.MsgRegistroExcluirSucesso;
                return RedirectToAction("Index");
            }
            catch
            {
                return View(model);
            }
        }
        public ActionResult Details(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.Idioma { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            return View(model);
        }

        [HttpGet]
        public FileResult ExportCSV()
        {
            var lista = new Models.Idioma().ToIEnumerable();
            var csv = "ID;Nome;Sigla;Padrão;Status\n";
            csv += lista.Aggregate(string.Empty,
                (seed, next) => seed +
                    next.Id.ToString() + ";" +
                    next.Nome + ";" +
                    next.Sigla + ";" +
                    next.Padrao + ";" +
                    next.Status +
                    "\n");

            var bytes = Encoding.GetEncoding("iso-8859-1").GetBytes(csv);
            var stream = new System.IO.MemoryStream(bytes);
            return File(stream, "text/csv", "DadosIdioma.csv");
        }
    }
}