﻿using AgileCompass.Helper;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Net;
using System.Text;
using System.Web.Mvc;

namespace AgileCompass.Controllers
{
    [VenomAuthorize(true)]
    public class ItemController : BaseController
    {

        public ActionResult Index(int? ResultadoId, string search, int? page)
        {
            if (ResultadoId == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);

            var obj = new Models.Item { ResultadoId = (int)ResultadoId };
            obj.Resultado = new Models.Resultado { Id = (int)ResultadoId, Status = true }.Find();
            if (obj.Resultado == null)
                return HttpNotFound();

            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = (string.IsNullOrEmpty(search)) ? new Models.Item().Where(x => x.ResultadoId == ResultadoId).ToList() : new Models.Item().SearchByString(search, ResultadoId);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = page ?? 1;
            ViewBag.QtdPorPagina = qtd;

            source = source.Skip(((int)(page ?? 1) - 1) * qtd);
            obj.ResultSet = source.Take(qtd);
            return View(obj);
        }

        [HttpPost]
        public ActionResult Index(int? ResultadoId, string search)
        {
            if (ResultadoId == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);

            var obj = new Models.Item { ResultadoId = (int)ResultadoId };
            obj.Resultado = new Models.Resultado { Id = (int)ResultadoId, Status = true }.Find();
            if (obj.Resultado == null)
                return HttpNotFound();

            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = new Models.Item().SearchByString(search, ResultadoId);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = 1;
            ViewBag.QtdPorPagina = qtd;

            obj.ResultSet = source.Take(qtd);
            return View(obj);
        }

        public ActionResult Create(int? ResultadoId)
        {
            if (ResultadoId == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);

            var model = new Models.Item { ResultadoId = (int)ResultadoId };
            model.Resultado = new Models.Resultado { Id = (int)ResultadoId }.Find();
            if (model.Resultado == null)
                return HttpNotFound();

            CarregarComboIdioma();
            return View(model);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Models.Item model)
        {
            try
            {
                model.Resultado = new Models.Resultado { Id = model.ResultadoId }.Find();
                model.DataCadastro = DateTime.Now;
                if (ModelState.IsValid && model.Save())
                {
                    Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                    return RedirectToAction("Index", new { resultadoId = model.ResultadoId });
                }

                model.IdiomaItem = CarregarItensIdioma(model.IdiomaItem);
                CarregarComboIdioma();
                return View(model);
            }
            catch (Exception ex)
            {
                model.IdiomaItem = CarregarItensIdioma(model.IdiomaItem);
                CarregarComboIdioma();
                return View(model);
            }
        }

        public ActionResult Edit(int? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            var obj = new Models.Item { Id = (int)id }.Find();
            if (obj == null)
            {
                return HttpNotFound();
            }
            CarregarComboIdioma();
            return View(obj);
        }

        [HttpPost]
        public ActionResult Edit(Models.Item model)
        {
            try
            {
                model.Resultado = new Models.Resultado { Id = model.ResultadoId }.Find();
                if (!ModelState.IsValid)
                {
                    model.IdiomaItem = CarregarItensIdioma(model.IdiomaItem);
                    CarregarComboIdioma();
                    return View(model);
                }
                var items = new Models.IdiomaItem().Where(x => x.ItemId == model.Id).ToList();
                foreach (var item in items)
                    item.Delete();
                foreach (var item in model.IdiomaItem)
                    item.Save();

                model.Save();
                Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                return RedirectToAction("Index", new { resultadoId = model.ResultadoId });
            }
            catch
            {
                model.IdiomaItem = CarregarItensIdioma(model.IdiomaItem);
                CarregarComboIdioma();
                return View(model);
            }
        }

        public ActionResult Delete(int? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            var obj = new Models.Item { Id = (int)id }.Find();
            if (obj == null)
            {
                return HttpNotFound();
            }
            return View(obj);
        }

        [HttpPost]
        public ActionResult Delete(Models.Item model)
        {
            try
            {
                if (!ModelState.IsValid)
                    return View(model);

                if ((model.GetDbContext()).Avaliacao.Where(x => x.AvaliacaoItem.Any(y => y.ItemId == model.Id)).Count() < 1)
                {
                    var items = new Models.IdiomaItem().Where(x => x.ItemId == model.Id).ToList();
                    foreach (var item in items)
                        item.Delete();

                    new Models.Item().Where(x => x.Id == model.Id).Single().Delete();
                }
                else
                {
                    ViewBag.Mensagem = Resource.MsgExcluirDependencia;
                    return View(new Models.Item { Id = model.Id }.Find());
                }

                Session["Mensagem"] = Resource.MsgRegistroExcluirSucesso;
                return RedirectToAction("Index", new { resultadoId = model.ResultadoId });
            }
            catch
            {
                return View(model);
            }
        }

        public ActionResult Details(int? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            var obj = new Models.Item { Id = (int)id }.Find();
            if (obj == null)
            {
                return HttpNotFound();
            }
            return View(obj);
        }

        [HttpGet]
        public FileResult ExportCSV(int? ResultadoId)
        {
            var lista = new Models.Item().Where(x => x.ResultadoId == (int)ResultadoId).ToList();
            var csv = "ID;Resultado;Descrição;Data de Cadastro;Status\n";
            csv += lista.Aggregate(string.Empty,
                (seed, next) => seed +
                    next.Id.ToString() + ";" +
                    next.ResultadoId.ToString() + ";" +
                    next.IdiomaItemDefault.Descricao + ";" +
                    next.DataCadastro + ";" +
                    next.Status +
                    "\n");

            var bytes = Encoding.GetEncoding("iso-8859-1").GetBytes(csv);
            var stream = new System.IO.MemoryStream(bytes);
            return File(stream, "text/csv", "DadosItem.csv");
        }

        void CarregarComboIdioma()
        {
            var data = new Models.Idioma().Where(x => x.Status).ToList();
            ViewBag.IdiomaId = new SelectList(data, "Id", "Nome");
        }

        // ### Helper Methods ###

        List<Models.IdiomaItem> CarregarItensIdioma(List<Models.IdiomaItem> source)
        {
            var data = new Models.Idioma().ToIEnumerable();
            source = source.Select(x => new Models.IdiomaItem
            {
                IdiomaId = x.IdiomaId,
                Descricao = x.Descricao,
                Idioma = data.Where(y => y.Id == x.IdiomaId).Single()
            }).ToList();
            return source;
        }
    }
}