﻿using AgileCompass.Helper;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Net;
using System.Text;
using System.Web.Mvc;

namespace AgileCompass.Controllers
{
    [VenomAuthorize(true)]
    public class ItemQuadroController : BaseController
    {
        public ActionResult Index(int? quadroId, string search, int? page)
        {
            if (quadroId == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);

            var obj = new Models.ItemQuadro { QuadroId = (int)quadroId };
            obj.Quadro = new Models.Quadro { Id = (int)quadroId, Status = true }.Find();
            if (obj.Quadro == null)
                return HttpNotFound();

            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = (string.IsNullOrEmpty(search)) ? new Models.ItemQuadro().Where(x => x.QuadroId == (int)quadroId).ToList() : new Models.ItemQuadro().SearchByString(search, quadroId);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = page ?? 1;
            ViewBag.QtdPorPagina = qtd;

            source = source.Skip(((int)(page ?? 1) - 1) * qtd);
            obj.ResultSet = source.Take(qtd);
            return View(obj);
        }

        [HttpPost]
        public ActionResult Index(int? quadroId, string search)
        {
            if (quadroId == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);

            var obj = new Models.ItemQuadro { QuadroId = (int)quadroId };
            obj.Quadro = new Models.Quadro { Id = (int)quadroId, Status = true }.Find();
            if (obj.Quadro == null)
                return HttpNotFound();

            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = new Models.ItemQuadro().SearchByString(search, quadroId);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = 1;
            ViewBag.QtdPorPagina = qtd;

            obj.ResultSet = source.Take(qtd);
            return View(obj);
        }

        public ActionResult Create(int? quadroId)
        {
            if (quadroId == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);

            var model = new Models.ItemQuadro { QuadroId = (int)quadroId };
            model.Quadro = new Models.Quadro { Id = (int)quadroId }.Find();
            if (model.Quadro == null)
                return HttpNotFound();

            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            CarregarComboIdioma();
            return View(model);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Models.ItemQuadro model)
        {
            try
            {
                model.Quadro = new Models.Quadro { Id = model.QuadroId }.Find();
                var itensPorQuadro = (model.GetDbContext()).ItemQuadro.Where(x => x.QuadroId == model.QuadroId).ToList().Count();
                if (itensPorQuadro < 4)
                {
                    if (ModelState.IsValid && model.Save())
                    {
                        Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                        return RedirectToAction("Index", new { quadroId = model.QuadroId });
                    }
                }
                else
                    ViewBag.Mensagem = Resource.MsgMaximo4Itens;
                model.IdiomaItemQuadro = CarregarItensIdioma(model.IdiomaItemQuadro);
                CarregarComboIdioma();
                return View(model);
            }
            catch (Exception ex)
            {
                model.IdiomaItemQuadro = CarregarItensIdioma(model.IdiomaItemQuadro);
                CarregarComboIdioma();
                return View(model);
            }
        }

        public ActionResult Edit(int? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            var obj = new Models.ItemQuadro { Id = (int)id }.Find();
            if (obj == null)
            {
                return HttpNotFound();
            }
            CarregarComboIdioma();
            return View(obj);
        }

        [HttpPost]
        public ActionResult Edit(Models.ItemQuadro model)
        {
            try
            {
                model.Quadro = new Models.Quadro { Id = model.QuadroId }.Find();
                if (!ModelState.IsValid)
                {
                    model.IdiomaItemQuadro = CarregarItensIdioma(model.IdiomaItemQuadro);
                    CarregarComboIdioma();
                    return View(model);
                }
                var items = new Models.IdiomaItemQuadro().Where(x => x.ItemQuadroId == model.Id).ToList();
                foreach (var item in items)
                    item.Delete();
                foreach (var item in model.IdiomaItemQuadro)
                    item.Save();

                model.Save();
                Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                return RedirectToAction("Index", new { quadroId = model.QuadroId });
            }
            catch
            {
                model.IdiomaItemQuadro = CarregarItensIdioma(model.IdiomaItemQuadro);
                CarregarComboIdioma();
                return View(model);
            }
        }

        public ActionResult Delete(int? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            var obj = new Models.ItemQuadro { Id = (int)id }.Find();
            if (obj == null)
            {
                return HttpNotFound();
            }
            return View(obj);
        }

        [HttpPost]
        public ActionResult Delete(Models.ItemQuadro model)
        {
            try
            {
                if (!ModelState.IsValid)
                    return View(model);

                var items = new Models.IdiomaItemQuadro().Where(x => x.ItemQuadroId == model.Id).ToList();
                foreach (var item in items)
                    item.Delete();

                new Models.ItemQuadro().Where(x => x.Id == model.Id).Single().Delete();

                Session["Mensagem"] = Resource.MsgRegistroExcluirSucesso;
                return RedirectToAction("Index", new { quadroId = model.QuadroId });
            }
            catch
            {
                return View(model);
            }
        }

        public ActionResult Details(int? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            var obj = new Models.ItemQuadro { Id = (int)id }.Find();
            if (obj == null)
            {
                return HttpNotFound();
            }
            return View(obj);
        }

        [HttpGet]
        public FileResult ExportCSV(int? quadroId)
        {
            var lista = new Models.ItemQuadro().Where(x => x.QuadroId == (int)quadroId).ToList();
            var csv = "ID;Quadro;Nome;Status\n";
            csv += lista.Aggregate(string.Empty,
                (seed, next) => seed +
                    next.Id.ToString() + ";" +
                    next.QuadroId.ToString() + ";" +
                    next.IdiomaItemQuadroDefault.Nome + ";" +
                    next.Status +
                    "\n");

            var bytes = Encoding.GetEncoding("iso-8859-1").GetBytes(csv);
            var stream = new System.IO.MemoryStream(bytes);
            return File(stream, "text/csv", "DadosItemQuadro.csv");
        }

        void CarregarComboIdioma()
        {
            var data = new Models.Idioma().Where(x => x.Status).ToList();
            ViewBag.IdiomaId = new SelectList(data, "Id", "Nome");
        }

        // ### Helper Methods ###

        List<Models.IdiomaItemQuadro> CarregarItensIdioma(List<Models.IdiomaItemQuadro> source)
        {
            var data = new Models.Idioma().ToIEnumerable();
            source = source.Select(x => new Models.IdiomaItemQuadro
            {
                IdiomaId = x.IdiomaId,
                Nome = x.Nome,
                Idioma = data.Where(y => y.Id == x.IdiomaId).Single()
            }).ToList();
            return source;
        }
    }
}