﻿using AgileCompass.Helper;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Net;
using System.Text;
using System.Web.Mvc;

namespace AgileCompass.Controllers
{
    [VenomAuthorize(true)]
    public class PraticaController : BaseController
    {
        // GET: Pratica
        public ActionResult Index(string search, int? page)
        {
            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = (string.IsNullOrEmpty(search)) ? new Models.Pratica().ToIEnumerable() : new Models.Pratica().SearchByString(search);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = page ?? 1;
            ViewBag.QtdPorPagina = qtd;

            source = source.Skip(((int)(page ?? 1) - 1) * qtd);
            return View(source.Take(qtd));
        }

        [HttpPost]
        public ActionResult Index(string search)
        {
            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = new Models.Pratica().SearchByString(search);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = 1;
            ViewBag.QtdPorPagina = qtd;

            return View(source.Take(qtd));
        }

        public ActionResult Create()
        {
            CarregarComboIdioma();
            return View(new Models.Pratica());
        }
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Models.Pratica model)
        {
            try
            {
                if (ModelState.IsValid && model.Save())
                {
                    Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                    return RedirectToAction("Index");
                }

                model.IdiomaPratica = CarregarItensIdioma(model.IdiomaPratica);
                CarregarComboIdioma();
                return View(model);
            }
            catch (Exception)
            {
                model.IdiomaPratica = CarregarItensIdioma(model.IdiomaPratica);
                CarregarComboIdioma();
                return View(model);
            }
        }
        public ActionResult Edit(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.Pratica { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            CarregarComboIdioma();
            return View(model);
        }
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(Models.Pratica model)
        {
            try
            {
                if (!ModelState.IsValid)
                {
                    model.IdiomaPratica = CarregarItensIdioma(model.IdiomaPratica);
                    CarregarComboIdioma();
                    return View(model);
                }
                var items = new Models.IdiomaPratica().Where(x => x.PraticaId == model.Id).ToList();
                foreach (var item in items)
                    item.Delete();
                foreach (var item in model.IdiomaPratica)
                    item.Save();

                model.Save();
                Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                return RedirectToAction("Index");
            }
            catch
            {
                model.IdiomaPratica = CarregarItensIdioma(model.IdiomaPratica);
                CarregarComboIdioma();
                return View(model);
            }
        }
        public ActionResult Delete(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.Pratica { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            return View(model);
        }
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Delete(Models.Pratica model)
        {
            try
            {
                if (!ModelState.IsValid)
                    return View(model);

                if ((model.GetDbContext()).Equipe.Where(x => x.Pratica.Any(y => y.Id == model.Id)).Count() < 1)
                {
                    var items = new Models.IdiomaPratica().Where(x => x.PraticaId == model.Id).ToList();
                    foreach (var item in items)
                        item.Delete();

                    new Models.Pratica().Where(x => x.Id == model.Id).Single().Delete();
                }
                else
                {
                    ViewBag.Mensagem = Resource.MsgExcluirDependencia;
                    return View(new Models.Pratica { Id = model.Id }.Find());
                }

                Session["Mensagem"] = Resource.MsgRegistroExcluirSucesso;
                return RedirectToAction("Index");
            }
            catch
            {
                return View(model);
            }
        }

        public ActionResult Details(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.Pratica { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            return View(model);
        }

        [HttpGet]
        public FileResult ExportCSV()
        {
            var lista = new Models.Pratica().ToIEnumerable();
            var csv = "ID;Nome;Status\n";
            csv += lista.Aggregate(string.Empty,
                (seed, next) => seed +
                    next.Id.ToString() + ";" +
                    next.IdiomaPraticaDefault.Nome + ";" +
                    next.Status +
                    "\n");

            var bytes = Encoding.GetEncoding("iso-8859-1").GetBytes(csv);
            var stream = new System.IO.MemoryStream(bytes);
            return File(stream, "text/csv", "DadosPratica.csv");
        }

        // ### Helper Methods ###

        void CarregarComboIdioma()
        {
            var data = new Models.Idioma().Where(x => x.Status).ToList();
            ViewBag.IdiomaId = new SelectList(data, "Id", "Nome");
        }

        List<Models.IdiomaPratica> CarregarItensIdioma(List<Models.IdiomaPratica> source)
        {
            var data = new Models.Idioma().ToIEnumerable();
            source = source.Select(x => new Models.IdiomaPratica
            {
                IdiomaId = x.IdiomaId,
                Nome = x.Nome,
                Idioma = data.Where(y => y.Id == x.IdiomaId).Single()
            }).ToList();
            return source;
        }
    }
}