﻿using AgileCompass.Helper;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Net;
using System.Text;
using System.Web.Mvc;

namespace AgileCompass.Controllers
{
    [VenomAuthorize(true)]
    public class ResultadoController : BaseController
    {
        public ActionResult Index(int? categoriaId, string search, int? page)
        {
            if (categoriaId == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);

            var obj = new Models.Resultado { CategoriaId = (int)categoriaId };
            obj.Categoria = new Models.Categoria { Id = (int)categoriaId, Status = true }.Find();
            if (obj.Categoria == null)
                return HttpNotFound();

            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = (string.IsNullOrEmpty(search)) ? new Models.Resultado().Where(x => x.CategoriaId == (int)categoriaId).ToList() : new Models.Resultado().SearchByString(search, categoriaId);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = page ?? 1;
            ViewBag.QtdPorPagina = qtd;

            source = source.Skip(((int)(page ?? 1) - 1) * qtd);
            obj.ResultSet = source.Take(qtd);
            return View(obj);
        }

        [HttpPost]
        public ActionResult Index(int? categoriaId, string search)
        {
            if (categoriaId == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);

            var obj = new Models.Resultado { CategoriaId = (int)categoriaId };
            obj.Categoria = new Models.Categoria { Id = (int)categoriaId, Status = true }.Find();
            if (obj.Categoria == null)
                return HttpNotFound();

            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;

            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = new Models.Resultado().SearchByString(search, categoriaId);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = 1;
            ViewBag.QtdPorPagina = qtd;

            obj.ResultSet = source.Take(qtd);
            return View(obj);
        }

        public ActionResult Create(int? categoriaId)
        {
            if (categoriaId == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);

            var model = new Models.Resultado { CategoriaId = (int)categoriaId };
            model.Categoria = new Models.Categoria { Id = (int)categoriaId }.Find();
            if (model.Categoria == null)
                return HttpNotFound();

            CarregarComboIdioma();
            return View(model);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Models.Resultado model)
        {
            try
            {
                model.Categoria = new Models.Categoria { Id = model.CategoriaId }.Find();
                model.DataCadastro = DateTime.Now;
                if (ModelState.IsValid && model.Save())
                {
                    Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                    return RedirectToAction("Index", new { categoriaId = model.CategoriaId });
                }

                model.IdiomaResultado = CarregarItensIdioma(model.IdiomaResultado);
                CarregarComboIdioma();
                return View(model);
            }
            catch (Exception ex)
            {
                model.IdiomaResultado = CarregarItensIdioma(model.IdiomaResultado);
                CarregarComboIdioma();
                return View(model);
            }
        }

        public ActionResult Edit(int? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            var obj = new Models.Resultado { Id = (int)id }.Find();
            if (obj == null)
            {
                return HttpNotFound();
            }
            CarregarComboIdioma();
            return View(obj);
        }

        [HttpPost]
        public ActionResult Edit(Models.Resultado model)
        {
            try
            {
                model.Categoria = new Models.Categoria { Id = model.CategoriaId }.Find();
                if (!ModelState.IsValid)
                {
                    model.IdiomaResultado = CarregarItensIdioma(model.IdiomaResultado);
                    CarregarComboIdioma();
                    return View(model);
                }
                var items = new Models.IdiomaResultado().Where(x => x.ResultadoId == model.Id).ToList();
                foreach (var item in items)
                    item.Delete();
                foreach (var item in model.IdiomaResultado)
                    item.Save();

                model.Save();
                Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                return RedirectToAction("Index", new { categoriaId = model.CategoriaId });
            }
            catch
            {
                model.IdiomaResultado = CarregarItensIdioma(model.IdiomaResultado);
                CarregarComboIdioma();
                return View(model);
            }
        }

        public ActionResult Delete(int? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            var obj = new Models.Resultado { Id = (int)id }.Find();
            if (obj == null)
            {
                return HttpNotFound();
            }
            return View(obj);
        }

        [HttpPost]
        public ActionResult Delete(Models.Resultado model)
        {
            try
            {
                if (!ModelState.IsValid)
                    return View(model);

                if ((model.GetDbContext()).Resultado.Where(x => x.Item.Any(y => y.ResultadoId == model.Id)).Count() < 1)
                {
                    var items = new Models.IdiomaResultado().Where(x => x.ResultadoId == model.Id).ToList();
                    foreach (var item in items)
                        item.Delete();

                    new Models.Resultado().Where(x => x.Id == model.Id).Single().Delete();
                }
                else
                {
                    ViewBag.Mensagem = Resource.MsgExcluirDependencia;
                    return View(new Models.Resultado { Id = model.Id }.Find());
                }

                Session["Mensagem"] = Resource.MsgRegistroExcluirSucesso;
                return RedirectToAction("Index", new { categoriaId = model.CategoriaId });
            }
            catch
            {
                return View(model);
            }
        }

        public ActionResult Details(int? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            var obj = new Models.Resultado { Id = (int)id }.Find();
            if (obj == null)
            {
                return HttpNotFound();
            }
            return View(obj);
        }

        [HttpGet]
        public FileResult ExportCSV(int? categoriaId)
        {
            var lista = new Models.Resultado().Where(x => x.CategoriaId == (int)categoriaId).ToList();
            var csv = "ID;Categoria;Nome;Descrição;Data de Cadstro;Status\n";
            csv += lista.Aggregate(string.Empty,
                (seed, next) => seed +
                    next.Id.ToString() + ";" +
                    next.CategoriaId.ToString() + ";" +
                    next.IdiomaResultadoDefault.Nome + ";" +
                    next.IdiomaResultadoDefault.Descricao + ";" +
                    next.DataCadastro + ";" +
                    next.Status +
                    "\n");

            var bytes = Encoding.GetEncoding("iso-8859-1").GetBytes(csv);
            var stream = new System.IO.MemoryStream(bytes);
            return File(stream, "text/csv", "DadosResultado.csv");
        }

        void CarregarComboIdioma()
        {
            var data = new Models.Idioma().Where(x => x.Status).ToList();
            ViewBag.IdiomaId = new SelectList(data, "Id", "Nome");
        }

        // ### Helper Methods ###

        List<Models.IdiomaResultado> CarregarItensIdioma(List<Models.IdiomaResultado> source)
        {
            var data = new Models.Idioma().ToIEnumerable();
            source = source.Select(x => new Models.IdiomaResultado
            {
                IdiomaId = x.IdiomaId,
                Nome = x.Nome,
                Descricao = x.Descricao,
                Idioma = data.Where(y => y.Id == x.IdiomaId).Single()
            }).ToList();
            return source;
        }
    }
}