﻿using System.Web.Mvc;
using System.Linq;
using System;
using System.Collections.Generic;
using System.Net;
using AgileCompass.Helper;
using System.Text;
using AgileCompass.Models;
using System.Configuration;

namespace AgileCompass.Controllers
{
    [VenomAuthorize]
    public class SugestaoMelhoriaController : BaseController
    {
        [VenomAuthorize(true)]
        public ActionResult Index(string search, int? page)
        {
            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = (string.IsNullOrEmpty(search)) ? new Models.SugestaoMelhoria().ToIEnumerable() : new Models.SugestaoMelhoria().SearchByString(search);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = page ?? 1;
            ViewBag.QtdPorPagina = qtd;

            source = source.Skip(((int)(page ?? 1) - 1) * qtd);
            return View(source.Take(qtd));
        }

        [VenomAuthorize(true)]
        [HttpPost]
        public ActionResult Index(string search)
        {
            int qtd = Convert.ToInt32(ConfigurationManager.AppSettings["QtdItensPagina"].ToString() ?? "10");
            var source = new Models.SugestaoMelhoria().SearchByString(search);

            ViewBag.Pesquisa = search;
            ViewBag.QuantidadeTotal = source.Count();
            ViewBag.PaginaAtual = 1;
            ViewBag.QtdPorPagina = qtd;

            return View(source.Take(qtd));
        }

        public ActionResult Create()
        {
            ViewBag.Mensagem = Session["Mensagem"];
            Session["Mensagem"] = string.Empty;
            return View();
        }
        [HttpPost]
        public ActionResult Create([Bind(Exclude = "Id, DataCadastro, Equipe")]SugestaoMelhoria obj)
        {
            obj.DataCadastro = DateTime.Now;
            obj.Descricao = WebUtility.HtmlDecode(obj.Descricao);
            var manager = new Helper.AuthenticationManager();
            obj.Equipe = manager.GetEquipe();
            ModelState.Remove("Equipe");
            if (ModelState.IsValid && obj.Save())
            {
                Session["Mensagem"] = Resource.MsgRegistroSalvoSucesso;
                return RedirectToAction("Create");
            }
            return View(obj);
        }

        [VenomAuthorize(true)]
        public ActionResult Details(int? id)
        {
            if (id == null)
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            var model = new Models.SugestaoMelhoria { Id = (int)id }.Find();
            if (model == null)
                return HttpNotFound();

            return View(model);
        }

        [VenomAuthorize(true)]
        [HttpGet]
        public FileResult ExportCSV()
        {
            var lista = new Models.SugestaoMelhoria().ToIEnumerable();
            var csv = "ID;Equipe;Data de Cadastro;Descrição\n";
            csv += lista.Aggregate(string.Empty,
                (seed, next) => seed +
                    next.Id.ToString() + ";" +
                    next.Equipe.Id.ToString() + ";" +
                    next.DataCadastro + ";" +
                    next.Descricao +
                    "\n");

            var bytes = Encoding.GetEncoding("iso-8859-1").GetBytes(csv);
            var stream = new System.IO.MemoryStream(bytes);
            return File(stream, "text/csv", "DadosSugestaoMelhoria.csv");
        }
    }
}