﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Net.Mail;
using System.Web;

namespace AgileCompass.Helper
{
    public class EmailHelper
    {
        /// <summary>
        /// Método responsável pelo envio de e-mails do sistema 
        /// Necessário configuração de SMTP, conta e senha no web.config 
        /// </summary>
        /// <param name="to">Array de string contendo os e-mails destinatários</param>
        /// <param name="subject">Assunto do e-mail</param>
        /// <param name="body">Corpo do e-mail</param>
        /// <param name="isBodyHtml">O corpo do e-mail é um HTML?</param>
        public static void Send(string[] to, string subject, string body, bool isBodyHtml)
        {
            var smtp = ConfigurationManager.AppSettings["AgileCompassMailSMTP"];
            var account = ConfigurationManager.AppSettings["AgileCompassMailAccount"];
            var password = ConfigurationManager.AppSettings["AgileCompassMailPassword"];

            using (var client = new SmtpClient(smtp))
            {
                var mail = new MailMessage()
                {
                    From = new MailAddress(account),
                    Subject = subject,
                    IsBodyHtml = isBodyHtml,
                    Body = body
                };

                foreach (var item in to)
                    mail.To.Add(new MailAddress(item));

                client.Port = 587;
                client.UseDefaultCredentials = false;
                client.Credentials = new System.Net.NetworkCredential(account, password);
                client.EnableSsl = true;
                client.Send(mail);
            }
        }

        public static string EmailContato(Models.ContatoResposta contatoResposta)
        {
            var template = @"
                <!DOCTYPE html>
                <html xmlns='http://www.w3.org/1999/xhtml'>
                <head>
	                <meta http-equiv='Content-Type' content='text/html; charset=UTF-8' />
	                <title>{0}</title>
                </head>
                <body>
	                <div style=""width: 620px; color:#000113; font-family:'Arial'"">
		                <div style=""padding:15px; background:#ffae3f; border-bottom:5px solid #000113;"">
			                <table style=""width:590px;"">
				                <tr>
					                <td style=""width:80px;""><img alt=""{0}"" src=""http://agilecompass.net/content/img/logoEmail.jpg"" /></td>
					                <td style=""text-align:center;""><h1 style=""font-size:22px;"">{1}</h1></td>
				                </tr>
			                </table>
		                </div>
		                <div style=""padding:15px;"">
                            <p><strong>{2}: </strong> <em>{3}</em></p>
			                <p>{4}</p>
		                </div>
		                <div style=""text-align:center; padding: 25px 15px; background:#F5F5F5; border-top:5px solid #000113;"">
			                <a href=""http://agilecompass.net/"" style=""color: #000113;"" target=""_blank""><em>agilecompass.net</em></a>
		                </div>
	                </div>
                </body>
                </html>";
            return string.Format(template, Resource.ViewBussolaAgil, Resource.EmailTituloContato, Resource.ModelContato, contatoResposta.Contato.Mensagem, contatoResposta.Mensagem);
        }

        public static string EmailRecuperarSenha(Models.Equipe equipe, string senha)
        {
            var template = @"
                <!DOCTYPE html>
                <html xmlns='http://www.w3.org/1999/xhtml'>
                <head>
	                <meta http-equiv='Content-Type' content='text/html; charset=UTF-8' />
	                <title>{0}</title>
                </head>
                <body>
	                <div style=""width: 620px; color:#000113; font-family:'Arial'"">
		                <div style=""padding:15px; background:#ffae3f; border-bottom:5px solid #000113;"">
			                <table style=""width:590px;"">
				                <tr>
					                <td style=""width:80px;""><img alt=""{0}"" src=""http://agilecompass.net/content/img/logoEmail.jpg"" /></td>
					                <td style=""text-align:center;""><h1 style=""font-size:22px;"">{1}</h1></td>
				                </tr>
			                </table>
		                </div>
		                <div style=""padding:15px;"">
			                <p>{2}</p>
			                <p><strong>{3}:</strong> <em>{4}</em></p>
			                <p><strong>{5}:</strong> <em>{6}</em></p>
		                </div>
		                <div style=""text-align:center; padding: 25px 15px; background:#F5F5F5; border-top:5px solid #000113;"">
			                <a href=""http://agilecompass.net/"" style=""color: #000113;"" target=""_blank""><em>agilecompass.net</em></a>
		                </div>
	                </div>
                </body>
                </html>";
            return string.Format(template, Resource.ViewBussolaAgil, Resource.EmailTituloRecuperarSenha, Resource.EmailRecuperarSenha, Resource.CampoLogin, equipe.Email, Resource.CampoSenha, senha);
        }
    }
}