﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Newtonsoft.Json;

namespace AgileCompass.Helper.Trello
{
    public class TrelloHelper
    {
        public string ApplicationKey { get; set; }
        public string Token { get; set; }
        public string BaseUrl { get; set; }

        public TrelloHelper(string appKey, string token)
        {
            ApplicationKey = appKey;
            Token = token;
            BaseUrl = "https://api.trello.com/1";
        }

        #region GET
        public IEnumerable<Board> GetBoards()
        {
            var apiPath = "/members/me/boards/";

            var request = System.Net.WebRequest.Create(BaseUrl + apiPath + "?key=" + ApplicationKey + "&token=" + Token);
            var response = request.GetResponse();
            var sr = new System.IO.StreamReader(response.GetResponseStream());
            var result = sr.ReadToEnd().Trim();

            return JsonConvert.DeserializeObject<IEnumerable<Board>>(result);
        }
        public IEnumerable<List> GetLists(string idBoard)
        {
            var request = System.Net.WebRequest.Create(BaseUrl + "/boards/" + idBoard + "/lists/?key=" + ApplicationKey + "&token=" + Token);
            var response = request.GetResponse();
            var sr = new System.IO.StreamReader(response.GetResponseStream());
            var result = sr.ReadToEnd().Trim();

            return JsonConvert.DeserializeObject<IEnumerable<List>>(result);
        }
        public IEnumerable<Card> GetCards(string idList)
        {
            var request = System.Net.WebRequest.Create(BaseUrl + "/lists/" + idList + "/cards/?key=" + ApplicationKey + "&token=" + Token);
            var response = request.GetResponse();
            var sr = new System.IO.StreamReader(response.GetResponseStream());
            var result = sr.ReadToEnd().Trim();

            return JsonConvert.DeserializeObject<IEnumerable<Card>>(result);
        }
        #endregion

        #region POST
        public Board AddBoard(string name)
        {
            var request = System.Net.WebRequest.Create(BaseUrl + "/boards/?key=" + ApplicationKey + "&token=" + Token + "&name=" + name);
            request.Method = "POST";
            var response = request.GetResponse();
            var sr = new System.IO.StreamReader(response.GetResponseStream());
            var result = sr.ReadToEnd().Trim();

            return JsonConvert.DeserializeObject<Board>(result);
        }

        public List AddList(string idBoard, string name)
        {
            var request = System.Net.WebRequest.Create(BaseUrl + "/lists/?key=" + ApplicationKey + "&token=" + Token + "&idBoard=" + idBoard + "&name=" + name);
            request.Method = "POST";
            var response = request.GetResponse();
            var sr = new System.IO.StreamReader(response.GetResponseStream());
            var result = sr.ReadToEnd().Trim();

            return JsonConvert.DeserializeObject<List>(result);
        }

        public Card AddCard(string idList, string name, string description)
        {
            var request = System.Net.WebRequest.Create(BaseUrl + "/cards/?key=" + ApplicationKey + "&token=" + Token + "&idList=" + idList + "&name=" + name + "&desc=" + description);
            request.Method = "POST";
            var response = request.GetResponse();
            var sr = new System.IO.StreamReader(response.GetResponseStream());
            var result = sr.ReadToEnd().Trim();

            return JsonConvert.DeserializeObject<Card>(result);
        }

        #endregion

        #region PUT

        public Board UpdateBoard(Board board)
        {
            var request = System.Net.WebRequest.Create(BaseUrl + "/boards/" + board.Id + "/?key=" + ApplicationKey + "&token=" + Token + "&name=" + board.Name);
            request.Method = "PUT";
            var response = request.GetResponse();
            var sr = new System.IO.StreamReader(response.GetResponseStream());
            var result = sr.ReadToEnd().Trim();

            return JsonConvert.DeserializeObject<Board>(result);
        }
        public List UpdateList(List list)
        {
            var request = System.Net.WebRequest.Create(BaseUrl + "/lists/" + list.Id + "/?key=" + ApplicationKey + "&token=" + Token + "&name=" + list.Name);
            request.Method = "PUT";
            var response = request.GetResponse();
            var sr = new System.IO.StreamReader(response.GetResponseStream());
            var result = sr.ReadToEnd().Trim();

            return JsonConvert.DeserializeObject<List>(result);
        }
        public Card UpdateCard(Card card)
        {
            var request = System.Net.WebRequest.Create(BaseUrl + "/cards/" + card.Id + "/?key=" + ApplicationKey + "&token=" + Token + "&name=" + card.Name);
            request.Method = "PUT";
            var response = request.GetResponse();
            var sr = new System.IO.StreamReader(response.GetResponseStream());
            var result = sr.ReadToEnd().Trim();

            return JsonConvert.DeserializeObject<Card>(result);
        }

        #endregion
    }
}
