﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using AgileCompass.Helper.Trello;
using Newtonsoft.Json;

namespace AgileCompass.Helper
{
    public class AvaliacaoTrelloHelper
    {
        Trello.TrelloHelper trello = null;

        public AvaliacaoTrelloHelper(string ApplicationKey,string token)
        {
            trello = new Trello.TrelloHelper(ApplicationKey, token);
        }

        public void SincronizarAvaliacao(Models.Avaliacao avaliacao)
        {
            if (string.IsNullOrEmpty(avaliacao.JsonBoard))
                return;
            var boardId = GetBoardId("Agile Compass");
            var listId = GetListId(boardId, "n" + avaliacao.Id, Resource.ModelAvaliacao + " n" + avaliacao.Id);
            UpdateList(listId, Resource.ModelAvaliacao + " n" + avaliacao.Id);
            var cardViewList = JsonConvert.DeserializeObject<IEnumerable<CardView>>(avaliacao.JsonBoard);
            foreach (var item in cardViewList)
            {
                var cardId = GetCardId(listId, "n" + item.Id, item.Texto + " n" + item.Id);
                UpdateCard(cardId, item.Texto + " n" + item.Id);
            }
        }

        string GetBoardId(string name)
        {
            var board = trello.GetBoards().Where(x => x.Name == name && !x.Closed).FirstOrDefault();
            if (board == null)
                return trello.AddBoard(name).Id;
            else
                return board.Id;
        }

        string GetListId(string boardId, string selector, string name)
        {
            var list = trello.GetLists(boardId).Where(x => x.Name.EndsWith(selector) && !x.Closed).FirstOrDefault();
            if (list == null)
                return trello.AddList(boardId, name).Id;
            else
                return list.Id;
        }

        string GetCardId(string listId, string selector, string name)
        {
            var card = trello.GetCards(listId).Where(x => x.Name.EndsWith(selector) && !x.Closed).FirstOrDefault();
            if (card == null)
                return trello.AddCard(listId, name, string.Empty).Id;
            else
                return card.Id;
        }

        void UpdateList(string listId, string name)
        {
            trello.UpdateList(new List { Id = listId, Name = name });
        }
        void UpdateCard(string cardId, string name)
        {
            trello.UpdateCard(new Card { Id = cardId, Name = name });
        }
    }
}