﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Data.Entity;
using System.Data.Entity.Migrations;
using System.ComponentModel.DataAnnotations;
using System.Linq.Expressions;
using AgileCompass.Data;

namespace AgileCompass.Models
{
    public abstract class Base<T> where T : class
    {
        protected AgileCompassData context = new AgileCompassDataManager().Current;
        public static implicit operator T(Base<T> source) { return source as T; }

        public IEnumerable<T> ResultSet { get; set; }

        Helper.AuthenticationManager _AuthenticationManager;
        protected Helper.AuthenticationManager AuthenticationManager
        {
            get { return _AuthenticationManager = _AuthenticationManager ?? new Helper.AuthenticationManager(); }
        }

        public virtual bool Save()
        {
            if (this == null)
                throw new NullReferenceException("Base TChild is null");
            context.Set<T>().AddOrUpdate(this);
            return context.SaveChanges() > 0;
        }

        public bool FindDelete()
        {
            if (this == null)
                throw new NullReferenceException("Base TChild is null");

            var obj = this.Find();

            context.Set<T>().Remove(obj);
            return context.SaveChanges() > 0;
        }

        public virtual bool Delete()
        {
            if (this == null)
                throw new NullReferenceException("Base TChild is null");

            context.Set<T>().Remove(this as T);
            return context.SaveChanges() > 0;
        }

        public T Find()
        {
            var keyProperty = typeof(T).GetProperties().Where(x => x.GetCustomAttributes(typeof(KeyAttribute), false).Length > 0).FirstOrDefault();
            if (keyProperty == null)
                throw new Exception("Key property not found.");
            var key = keyProperty.GetValue(this);
            if (key == null)
                return null;
            return context.Set<T>().Find(key);
        }

        public IEnumerable<T> ToIEnumerable()
        {
            return context.Set<T>().ToList();
        }
        public IQueryable<T> ToIQueryable()
        {
            return context.Set<T>().AsQueryable();
        }
        public IQueryable<T> Where(Expression<Func<T, bool>> predicate)
        {
            return context.Set<T>().Where(predicate);
        }
        public bool Any(Expression<Func<T, bool>> predicate)
        {
            return context.Set<T>().Any(predicate);
        }
        public int Count()
        {
            return context.Set<T>().Count();
        }
        public void SetAsEntityModified()
        {
            context.Entry(this as T).State = EntityState.Modified;
        }
        public AgileCompassData GetDbContext()
        {
            return this.context;
        }
    }
}
