﻿using AgileCompass.Attributes;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Web;

namespace AgileCompass.Models
{
    public class Categoria : Base<Categoria>
    {
        [Key]
        public int Id { get; set; }

        [Required(ErrorMessageResourceName = "MsgCampoObrigatorio", ErrorMessageResourceType = typeof(Resource))]
        [Display(Name = "CampoDataCadastro", ResourceType = typeof(Resource))]
        public DateTime DataCadastro { get; set; }

        [Display(Name = "CampoStatus", ResourceType = typeof(Resource))]
        public bool Status { get; set; }

        [DataType(DataType.Upload)]
        [Display(Name = "CampoImagem", ResourceType = typeof(Resource))]
        public string Imagem { get; set; }

        [DataType(DataType.Upload)]
        [Display(Name = "CampoImagemTese", ResourceType = typeof(Resource))]
        public string ImagemTese { get; set; }

        [Display(Name = "CampoIdioma", ResourceType = typeof(Resource))]
        public virtual List<IdiomaCategoria> IdiomaCategoria { get; set; }

        [Display(Name = "CampoResultado", ResourceType = typeof(Resource))]
        public virtual IList<Resultado> Resultado { get; set; }

        [NotMapped]
        public IdiomaCategoria IdiomaCategoriaDefault
        {
            get
            {
                var idiomaId = AuthenticationManager.IsAuthenticated ?
                    AuthenticationManager.GetEquipe().IdiomaId : 0;
                var idioma = HttpContext.Current.Session["Idioma"].ToString();
                return
                    IdiomaCategoria.Where(x => x.Idioma.Sigla.Contains(idioma)).SingleOrDefault() ??
                    IdiomaCategoria.Where(x => x.IdiomaId == idiomaId).SingleOrDefault() ??
                    IdiomaCategoria.Where(x => x.Idioma.Padrao).SingleOrDefault() ??
                    IdiomaCategoria.FirstOrDefault();
            }
        }

        public Categoria()
        {
            this.IdiomaCategoria = new List<IdiomaCategoria>();
        }

        public IEnumerable<Categoria> SearchByString(string value)
        {
            var idiomaId = AuthenticationManager.IsAuthenticated ? AuthenticationManager.GetEquipe().IdiomaId : 0;

            return this
                .Where(x =>
                    x.IdiomaCategoria.Where(y => y.IdiomaId == idiomaId || y.Idioma.Padrao || true).FirstOrDefault().Nome.Contains(value) ||
                    x.IdiomaCategoria.Where(y => y.IdiomaId == idiomaId || y.Idioma.Padrao || true).FirstOrDefault().Descricao.Contains(value)
                ).ToList();
        }
    }
}