﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Web;

namespace AgileCompass.Models
{
    public class Conteudo : Base<Conteudo>
    {
        [Key]
        public int Id { get; set; }

        [DataType(DataType.Upload)]
        [Display(Name = "CampoImagem", ResourceType = typeof(Resource))]
        public string Imagem { get; set; }

        [Required]
        [Display(Name = "CampoDataCadastro", ResourceType = typeof(Resource))]
        public DateTime DataCadastro { get; set; }

        [Required]
        [Display(Name = "CampoStatus", ResourceType = typeof(Resource))]
        public bool Status { get; set; }

        [Display(Name = "CampoOrdem", ResourceType = typeof(Resource))]
        public int Ordem { get; set; }

        [Required]
        [Display(Name = "CampoTipoConteudo", ResourceType = typeof(Resource))]
        public int TipoConteudoId { get; set; }
        public virtual TipoConteudo TipoConteudo { get; set; }

        [Display(Name = "CampoIdioma", ResourceType = typeof(Resource))]
        public virtual List<IdiomaConteudo> IdiomaConteudo { get; set; }

        [NotMapped]
        public IdiomaConteudo IdiomaConteudoDefault
        {
            get
            {
                var idiomaId = AuthenticationManager.IsAuthenticated ?
                    AuthenticationManager.GetEquipe().IdiomaId : 0;
                var idioma = HttpContext.Current.Session["Idioma"].ToString();
                return
                    IdiomaConteudo.Where(x => x.Idioma.Sigla.Contains(idioma)).SingleOrDefault() ??
                    IdiomaConteudo.Where(x => x.IdiomaId == idiomaId).SingleOrDefault() ??
                    IdiomaConteudo.Where(x => x.Idioma.Padrao).SingleOrDefault() ??
                    IdiomaConteudo.FirstOrDefault();
            }
        }

        public Conteudo()
        {
            this.IdiomaConteudo = new List<IdiomaConteudo>();
        }

        public IEnumerable<Conteudo> SearchByString(string value)
        {
            var idiomaId = AuthenticationManager.IsAuthenticated ? AuthenticationManager.GetEquipe().IdiomaId : 0;

            return this
                .Where(x =>
                    x.IdiomaConteudo.Where(y => y.IdiomaId == idiomaId || y.Idioma.Padrao || true).FirstOrDefault().Titulo.Contains(value) ||
                    x.IdiomaConteudo.Where(y => y.IdiomaId == idiomaId || y.Idioma.Padrao || true).FirstOrDefault().Subtitulo.Contains(value) ||
                    x.IdiomaConteudo.Where(y => y.IdiomaId == idiomaId || y.Idioma.Padrao || true).FirstOrDefault().ConteudoValue.Contains(value)
                ).ToList();
        }
    }
}