﻿using AgileCompass.Attributes;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;

namespace AgileCompass.Models
{
    public class Equipe : Base<Equipe>
    {
        [Key]
        public int Id { get; set; }

        [Required(ErrorMessageResourceName = "MsgCampoObrigatorio", ErrorMessageResourceType = typeof(Resource))]
        [MaxLength(80,ErrorMessageResourceName = "MsgMaxLength", ErrorMessageResourceType = typeof(Resource))]
        [Display(Name = "CampoNome", ResourceType = typeof(Resource))]
        public string Nome { get; set; }

        [Required(ErrorMessageResourceName = "MsgCampoObrigatorio", ErrorMessageResourceType = typeof(Resource))]
        [MaxLength(200, ErrorMessageResourceName = "MsgMaxLength", ErrorMessageResourceType = typeof(Resource))]
        [DataType(DataType.EmailAddress, ErrorMessageResourceName = "MsgEmailInvalido", ErrorMessageResourceType = typeof(Resource))]
        [Display(Name = "CampoEmail", ResourceType = typeof(Resource))]
        public string Email { get; set; }

        [Required(ErrorMessageResourceName = "MsgCampoObrigatorio", ErrorMessageResourceType = typeof(Resource))]
        [MaxLength(45, ErrorMessageResourceName = "MsgMaxLength", ErrorMessageResourceType = typeof(Resource))]
        [Display(Name = "CampoCidade", ResourceType = typeof(Resource))]
        public string Cidade { get; set; }

        [Required(ErrorMessageResourceName = "MsgCampoObrigatorio", ErrorMessageResourceType = typeof(Resource))]
        [Display(Name = "CampoNumIntegrante", ResourceType = typeof(Resource))]
        public int NumIntegrante { get; set; }

        [MaxLength(45, ErrorMessageResourceName = "MsgMaxLength", ErrorMessageResourceType = typeof(Resource))]
        [Display(Name = "CampoEmpresa", ResourceType = typeof(Resource))]
        public string Empresa { get; set; }

        [Required(ErrorMessageResourceName = "MsgCampoObrigatorio", ErrorMessageResourceType = typeof(Resource))]
        [Display(Name = "CampoDataInicioAgil", ResourceType = typeof(Resource))]
        public DateTime DataInicioAgil { get; set; }

        [Required(ErrorMessageResourceName = "MsgCampoObrigatorio", ErrorMessageResourceType = typeof(Resource))]
        [Display(Name = "CampoDataCadastro", ResourceType = typeof(Resource))]
        public DateTime DataCadastro { get; set; }

        [Display(Name = "CampoAdmin", ResourceType = typeof(Resource))]
        public bool Admin { get; set; }

        [Display(Name = "CampoStatus", ResourceType = typeof(Resource))]
        public bool Status { get; set; }

        [Required(ErrorMessageResourceName = "MsgCampoObrigatorio", ErrorMessageResourceType = typeof(Resource))]
        [Display(Name = "CampoPais", ResourceType = typeof(Resource))]
        public int PaisId { get; set; }

        [Display(Name = "CampoPais", ResourceType = typeof(Resource))]
        public virtual Pais Pais { get; set; }

        [Required(ErrorMessageResourceName = "MsgCampoObrigatorio", ErrorMessageResourceType = typeof(Resource))]
        [Display(Name = "CampoIdioma", ResourceType = typeof(Resource))]
        public int IdiomaId { get; set; }

        [Display(Name = "CampoIdioma", ResourceType = typeof(Resource))]
        public virtual Idioma Idioma { get; set; }

        [Required(ErrorMessageResourceName = "MsgCampoObrigatorio", ErrorMessageResourceType = typeof(Resource))]
        [DataType(DataType.Password)]
        [ExpressionType(System.Linq.Expressions.ExpressionType.Equal)]
        [Display(Name = "CampoSenha", ResourceType = typeof(Resource))]
        public string Senha { get; set; }

        public string TokenTrello { get; set; }

        [Display(Name = "CampoEquipeTipoSoftware", ResourceType = typeof(Resource))]
        public virtual ICollection<TipoSoftware> TipoSoftware { get; set; }

        [Display(Name = "CampoEquipePratica", ResourceType = typeof(Resource))]
        public virtual ICollection<Pratica> Pratica { get; set; }

        [Display(Name = "CampoEquipeMetodo", ResourceType = typeof(Resource))]
        public virtual ICollection<Metodo> Metodo { get; set; }

        public En.Status Autenticar()
        {
            var data = this.Where(x => x.Email.Equals(this.Email) && x.Senha.Equals(this.Senha) && x.Status).ToList();

            if (data.Count() > 0)
            {
                this.Id = data[0].Id;
                this.Nome = data[0].Nome;
                this.Idioma = data[0].Idioma;
            }

            return data.Count() > 0 ? En.Status.Sucesso : En.Status.Falha;
        }

        public Equipe()
        {
            this.Metodo = new HashSet<Metodo>();
            this.Pratica = new HashSet<Pratica>();
            this.TipoSoftware = new HashSet<TipoSoftware>();
        }

        public IEnumerable<Equipe> SearchByString(string value)
        {
            return this
                .Where(x =>
                    x.Nome.Contains(value) ||
                    x.Email.Contains(value)
                ).ToList();
        }
    }
}