﻿using AgileCompass.Attributes;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Web;

namespace AgileCompass.Models
{
    public class Item : Base<Item>
    {
        [Key]
        public int Id { get; set; }

        [Required(ErrorMessageResourceName = "MsgCampoObrigatorio", ErrorMessageResourceType = typeof(Resource))]
        [Display(Name = "CampoDataCadastro", ResourceType = typeof(Resource))]
        public DateTime DataCadastro { get; set; }

        [Display(Name = "CampoStatus", ResourceType = typeof(Resource))]
        public bool Status { get; set; }

        [Required(ErrorMessageResourceName = "MsgCampoObrigatorio", ErrorMessageResourceType = typeof(Resource))]
        [Display(Name = "CampoResultado", ResourceType = typeof(Resource))]
        public int ResultadoId { get; set; }
        public virtual Resultado Resultado { get; set; }

        [Display(Name = "CampoIdioma", ResourceType = typeof(Resource))]
        public virtual List<IdiomaItem> IdiomaItem { get; set; }

        [NotMapped]
        public IdiomaItem IdiomaItemDefault
        {
            get
            {
                var idiomaId = AuthenticationManager.IsAuthenticated ?
                    AuthenticationManager.GetEquipe().IdiomaId : 0;
                var idioma = HttpContext.Current.Session["Idioma"].ToString();
                return
                    IdiomaItem.Where(x => x.Idioma.Sigla.Contains(idioma)).SingleOrDefault() ??
                    IdiomaItem.Where(x => x.IdiomaId == idiomaId).SingleOrDefault() ??
                    IdiomaItem.Where(x => x.Idioma.Padrao).SingleOrDefault() ??
                    IdiomaItem.FirstOrDefault();
            }
        }

        public IdiomaItem GetIdiomaItemByIdioma(string idioma)
        {
            return
                IdiomaItem.Where(x => x.Idioma.Sigla.Contains(idioma)).SingleOrDefault() ??
                IdiomaItem.Where(x => x.Idioma.Padrao).SingleOrDefault() ??
                IdiomaItem.FirstOrDefault();

        }

        public Item()
        {
            this.IdiomaItem = new List<IdiomaItem>();
        }

        public IEnumerable<Item> SearchByString(string value, int? resultadoId)
        {
            var idiomaId = AuthenticationManager.IsAuthenticated ? AuthenticationManager.GetEquipe().IdiomaId : 0;

            return this
                .Where(x =>
                    x.IdiomaItem.Where(y => y.IdiomaId == idiomaId || y.Idioma.Padrao || true).FirstOrDefault().Descricao.Contains(value) &&
                    x.ResultadoId == resultadoId
                ).ToList();
        }
    }
}