﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Web;

namespace AgileCompass.Models
{
    public class ItemQuadro : Base<ItemQuadro>
    {
        [Key]
        public int Id { get; set; }

        [Display(Name = "CampoStatus", ResourceType = typeof(Resource))]
        public bool Status { get; set; }

        [Required(ErrorMessageResourceName = "MsgCampoObrigatorio", ErrorMessageResourceType = typeof(Resource))]
        [Display(Name = "CampoQuadro", ResourceType = typeof(Resource))]
        public int QuadroId { get; set; }
        public virtual Quadro Quadro { get; set; }

        [Display(Name = "CampoIdioma", ResourceType = typeof(Resource))]
        public virtual List<IdiomaItemQuadro> IdiomaItemQuadro { get; set; }

        [NotMapped]
        public IdiomaItemQuadro IdiomaItemQuadroDefault
        {
            get
            {
                var idiomaId = AuthenticationManager.IsAuthenticated ?
                    AuthenticationManager.GetEquipe().IdiomaId : 0;
                var idioma = HttpContext.Current.Session["Idioma"].ToString();
                return
                    IdiomaItemQuadro.Where(x => x.Idioma.Sigla.Contains(idioma)).SingleOrDefault() ??
                    IdiomaItemQuadro.Where(x => x.IdiomaId == idiomaId).SingleOrDefault() ??
                    IdiomaItemQuadro.Where(x => x.Idioma.Padrao).SingleOrDefault() ??
                    IdiomaItemQuadro.FirstOrDefault();
            }
        }

        public ItemQuadro()
        {
            this.IdiomaItemQuadro = new List<IdiomaItemQuadro>();
        }

        public IEnumerable<ItemQuadro> SearchByString(string value, int? quadroId)
        {
            var idiomaId = AuthenticationManager.IsAuthenticated ? AuthenticationManager.GetEquipe().IdiomaId : 0;

            return this
                .Where(x =>
                    x.IdiomaItemQuadro.Where(y => y.IdiomaId == idiomaId || y.Idioma.Padrao || true).FirstOrDefault().Nome.Contains(value) &&
                    x.Quadro.Id == quadroId
                ).ToList();
        }
    }
}