﻿using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Web;

namespace AgileCompass.Models
{
    public class Metodo : Base<Metodo>
    {
        [Key]
        public int Id { get; set; }

        [Display(Name = "CampoStatus", ResourceType = typeof(Resource))]
        public bool Status { get; set; }

        [Display(Name = "CampoIdioma", ResourceType = typeof(Resource))]
        public virtual List<IdiomaMetodo> IdiomaMetodo { get; set; }

        [Display(Name = "CampoEquipe", ResourceType = typeof(Resource))]
        public virtual ICollection<Equipe> Equipe { get; set; }

        [NotMapped]
        public IdiomaMetodo IdiomaMetodoDefault
        {
            get
            {
                var idiomaId = AuthenticationManager.IsAuthenticated ?
                    AuthenticationManager.GetEquipe().IdiomaId : 0;
                var idioma = HttpContext.Current.Session["Idioma"].ToString();
                return
                    IdiomaMetodo.Where(x => x.Idioma.Sigla.Contains(idioma)).SingleOrDefault() ??
                    IdiomaMetodo.Where(x => x.IdiomaId == idiomaId).SingleOrDefault() ??
                    IdiomaMetodo.Where(x => x.Idioma.Padrao).SingleOrDefault() ??
                    IdiomaMetodo.FirstOrDefault();
            }
        }

        public Metodo()
        {
            this.IdiomaMetodo = new List<IdiomaMetodo>();
        }

        public IEnumerable<Metodo> SearchByString(string value)
        {
            var idiomaId = AuthenticationManager.IsAuthenticated ? AuthenticationManager.GetEquipe().IdiomaId : 0;

            return this
                .Where(x => 
                    x.IdiomaMetodo.Where(y => y.IdiomaId == idiomaId || y.Idioma.Padrao || true).FirstOrDefault().Nome.Contains(value) ||
                    x.IdiomaMetodo.Where(y => y.IdiomaId == idiomaId || y.Idioma.Padrao || true).FirstOrDefault().Sigla.Contains(value)
                ).ToList();
        }
    }
}