﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Web;

namespace AgileCompass.Models
{ 
    public class Pais : Base<Pais>
    {
        [Key]
        public int Id { get; set; }

        [Display(Name = "CampoIdioma", ResourceType = typeof(Resource))]
        public virtual List<IdiomaPais> IdiomaPais { get; set; }
  
        [NotMapped]
        public IdiomaPais IdiomaPaisDefault
        {            
            get
            {
                var idiomaId = AuthenticationManager.IsAuthenticated ?
                    AuthenticationManager.GetEquipe().IdiomaId : 0;
                var idioma = HttpContext.Current.Session["Idioma"].ToString();
                return
                    IdiomaPais.Where(x => x.Idioma.Sigla.Contains(idioma)).SingleOrDefault() ??
                    IdiomaPais.Where(x => x.IdiomaId == idiomaId).SingleOrDefault() ??
                    IdiomaPais.Where(x => x.Idioma.Padrao).SingleOrDefault() ??
                    IdiomaPais.FirstOrDefault();
            }
        }

        public Pais()
        {
            this.IdiomaPais = new List<IdiomaPais>();
        }

        public IEnumerable<Pais> SearchByString(string value)
        {
            var idiomaId = AuthenticationManager.IsAuthenticated ? AuthenticationManager.GetEquipe().IdiomaId : 0;

            return this
                .Where(x =>
                    x.IdiomaPais.Where(y => y.IdiomaId == idiomaId || y.Idioma.Padrao || true).FirstOrDefault().Nome.Contains(value)
                ).ToList();
        }
    }
}