﻿using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Web;

namespace AgileCompass.Models
{
    public class Pratica : Base<Pratica>
    {
        [Key]
        public int Id { get; set; }

        [Display(Name = "CampoStatus", ResourceType = typeof(Resource))]
        public bool Status { get; set; }

        [Display(Name = "CampoIdioma", ResourceType = typeof(Resource))]
        public virtual List<IdiomaPratica> IdiomaPratica { get; set; }

        [Display(Name = "CampoEquipe", ResourceType = typeof(Resource))]
        public virtual ICollection<Equipe> Equipe { get; set; }

        [NotMapped]
        public IdiomaPratica IdiomaPraticaDefault
        {
            get
            {
                var idiomaId = AuthenticationManager.IsAuthenticated ?
                    AuthenticationManager.GetEquipe().IdiomaId : 0;
                var idioma = HttpContext.Current.Session["Idioma"].ToString();
                return
                    IdiomaPratica.Where(x => x.Idioma.Sigla.Contains(idioma)).SingleOrDefault() ??
                    IdiomaPratica.Where(x => x.IdiomaId == idiomaId).SingleOrDefault() ??
                    IdiomaPratica.Where(x => x.Idioma.Padrao).SingleOrDefault() ??
                    IdiomaPratica.FirstOrDefault();
            }
        }

        public Pratica()
        {
            this.IdiomaPratica = new List<IdiomaPratica>();
        }

        public IEnumerable<Pratica> SearchByString(string value)
        {
            var idiomaId = AuthenticationManager.IsAuthenticated ? AuthenticationManager.GetEquipe().IdiomaId : 0;

            return this
                .Where(x =>
                    x.IdiomaPratica.Where(y => y.IdiomaId == idiomaId || y.Idioma.Padrao || true).FirstOrDefault().Nome.Contains(value)
                ).ToList();
        }
    }
}