﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Web;

namespace AgileCompass.Models
{
    public class Quadro : Base<Quadro>
    {
        [Key]
        public int Id { get; set; }

        [Display(Name = "CampoStatus", ResourceType = typeof(Resource))]
        public bool Status { get; set; }

        [Display(Name = "CampoIdioma", ResourceType = typeof(Resource))]
        public virtual List<IdiomaQuadro> IdiomaQuadro { get; set; }

        [Display(Name = "CampoItemQuadro", ResourceType = typeof(Resource))]
        public virtual List<ItemQuadro> ItemQuadro { get; set; }

        [NotMapped]
        public IdiomaQuadro IdiomaQuadroDefault
        {
            get
            {
                var idiomaId = AuthenticationManager.IsAuthenticated ?
                    AuthenticationManager.GetEquipe().IdiomaId : 0;
                var idioma = HttpContext.Current.Session["Idioma"].ToString();
                return
                    IdiomaQuadro.Where(x => x.Idioma.Sigla.Contains(idioma)).SingleOrDefault() ??
                    IdiomaQuadro.Where(x => x.IdiomaId == idiomaId).SingleOrDefault() ??
                    IdiomaQuadro.Where(x => x.Idioma.Padrao).SingleOrDefault() ??
                    IdiomaQuadro.FirstOrDefault();
            }
        }

        public Quadro()
        {
            this.IdiomaQuadro = new List<IdiomaQuadro>();
        }

        public IEnumerable<Quadro> SearchByString(string value)
        {
            var idiomaId = AuthenticationManager.IsAuthenticated ? AuthenticationManager.GetEquipe().IdiomaId : 0;

            return this
                .Where(x =>
                    x.IdiomaQuadro.Where(y => y.IdiomaId == idiomaId || y.Idioma.Padrao || true).FirstOrDefault().Nome.Contains(value)
                ).ToList();
        }
    }
}