﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Web;

namespace AgileCompass.Models
{
    public class Resultado : Base<Resultado>
    {
        [Key]
        public int Id { get; set; }

        [Required(ErrorMessageResourceName = "MsgCampoObrigatorio", ErrorMessageResourceType = typeof(Resource))]
        [Display(Name = "CampoDataCadastro", ResourceType = typeof(Resource))]
        public DateTime DataCadastro { get; set; }

        [Display(Name = "CampoStatus", ResourceType = typeof(Resource))]
        public bool Status { get; set; }

        [Display(Name = "CampoMaduro", ResourceType = typeof(Resource))]
        public bool Maduro { get; set; }

        [Required(ErrorMessageResourceName = "MsgCampoObrigatorio", ErrorMessageResourceType = typeof(Resource))]
        [Display(Name = "CampoCategoria", ResourceType = typeof(Resource))]
        public int CategoriaId { get; set; }
        public virtual Categoria Categoria { get; set; }

        [Display(Name = "CampoIdioma", ResourceType = typeof(Resource))]
        public virtual List<IdiomaResultado> IdiomaResultado { get; set; }

        [Display(Name = "CampoItem", ResourceType = typeof(Resource))]
        public virtual IList<Item> Item { get; set; }

        [NotMapped]
        public IdiomaResultado IdiomaResultadoDefault
        {
            get
            {
                var idiomaId = AuthenticationManager.IsAuthenticated ?
                    AuthenticationManager.GetEquipe().IdiomaId : 0;
                var idioma = HttpContext.Current.Session["Idioma"].ToString();
                return
                    IdiomaResultado.Where(x => x.Idioma.Sigla.Contains(idioma)).SingleOrDefault() ??
                    IdiomaResultado.Where(x => x.IdiomaId == idiomaId).SingleOrDefault() ??
                    IdiomaResultado.Where(x => x.Idioma.Padrao).SingleOrDefault() ??
                    IdiomaResultado.FirstOrDefault();
            }
        }

        public Resultado()
        {
            this.IdiomaResultado = new List<IdiomaResultado>();
        }

        public IEnumerable<Resultado> SearchByString(string value, int? categoriaId)
        {
            var idiomaId = AuthenticationManager.IsAuthenticated ? AuthenticationManager.GetEquipe().IdiomaId : 0;

            return this
                .Where(x =>
                    (x.IdiomaResultado.Where(y => y.IdiomaId == idiomaId || y.Idioma.Padrao || true).FirstOrDefault().Nome.Contains(value) ||
                    x.IdiomaResultado.Where(y => y.IdiomaId == idiomaId || y.Idioma.Padrao || true).FirstOrDefault().Descricao.Contains(value)) &&
                    x.CategoriaId == categoriaId
                ).ToList();
        }
    }
}