﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Web;

namespace AgileCompass.Models
{
    public class TipoSoftware : Base<TipoSoftware>
    {
        [Key]
        public int Id { get; set; }

        [Display(Name = "CampoStatus", ResourceType = typeof(Resource))]
        public bool Status { get; set; }
        
        [Display(Name = "CampoIdioma", ResourceType = typeof(Resource))]
        public virtual List<IdiomaTipoSoftware> IdiomaTipoSoftware { get; set; }

        [Display(Name = "CampoEquipe", ResourceType = typeof(Resource))]
        public virtual ICollection<Equipe> Equipe { get; set; }

        [NotMapped]
        public IdiomaTipoSoftware IdiomaTipoSoftwareDefault
        {
            get
            {
                var idiomaId = AuthenticationManager.IsAuthenticated ?
                    AuthenticationManager.GetEquipe().IdiomaId : 0;
                var idioma = HttpContext.Current.Session["Idioma"].ToString();
                return
                    IdiomaTipoSoftware.Where(x => x.Idioma.Sigla.Contains(idioma)).SingleOrDefault() ??
                    IdiomaTipoSoftware.Where(x => x.IdiomaId == idiomaId).SingleOrDefault() ??
                    IdiomaTipoSoftware.Where(x => x.Idioma.Padrao).SingleOrDefault() ??
                    IdiomaTipoSoftware.FirstOrDefault();
            }
        }

        public TipoSoftware()
        {
            this.IdiomaTipoSoftware = new List<IdiomaTipoSoftware>();
        }

        public IEnumerable<TipoSoftware> SearchByString(string value)
        {
            var idiomaId = AuthenticationManager.IsAuthenticated ? AuthenticationManager.GetEquipe().IdiomaId : 0;

            return this
                .Where(x =>
                    x.IdiomaTipoSoftware.Where(y => y.IdiomaId == idiomaId || y.Idioma.Padrao || true).FirstOrDefault().Nome.Contains(value)
                ).ToList();
        }
    }
}